"""
@Author: kang.yang
@Date: 2023/8/21 16:38
"""
import kuto
from page.ocr_page import OcrPage


class TestOcrDemo(kuto.Case):
    """ocr识别demo"""

    def start(self):
        self.op = OcrPage(self.driver)

    def test_nanshan_wtt(self):
        self.op.searchBtn.click()
        self.op.searchInput.input("南山文体通")
        self.op.searchResult.click()
        self.op.schoolEntry.click()
        self.sleep(5)


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform='ios',
        device_id='00008101-000E646A3C29003A',
        pkg_name='com.tencent.xin'
    )







"""
@Author: kang.yang
@Date: 2023/9/14 14:43
"""
import subprocess
import time

import pyautogui

from kuto.utils.common import screenshot_util
from kuto.utils.log import logger


class MacDriver(object):

    def __init__(self, pkg_name: str = None):
        self.app_name = pkg_name

    @staticmethod
    def get_location():
        """获取鼠标位置坐标"""
        while True:
            time.sleep(3)
            x, y = pyautogui.position()
            print(x, y)

    def start_app(self):
        if self.app_name is None:
            raise KeyError("应用名不能为空")

        logger.info(f'启动应用: {self.app_name}')
        cmd = f"open -a '{self.app_name}'"
        subprocess.Popen(cmd, shell=True)

    def stop_app(self):
        if self.app_name is None:
            raise KeyError("应用名不能为空")

        logger.info(f'关闭应用: {self.app_name}')
        cmd = f"killall '{self.app_name}'"
        subprocess.Popen(cmd, shell=True)

    @staticmethod
    def locate(image_path: str):
        """根据图片获取坐标"""
        try:
            x, y = pyautogui.locateCenterOnScreen(image_path)
            return x / 2, y / 2
        except:
            return None

    @staticmethod
    def is_on_screen(x, y):
        """坐标是否在屏幕中"""
        return pyautogui.onScreen(x, y)

    @staticmethod
    def click(x, y):
        logger.info(f"点击: ({x}, {y})")
        pyautogui.click(x, y)

    @staticmethod
    def input(text: str):
        logger.info(f"输入: {text}")
        pyautogui.write(text)

    @staticmethod
    def enter():
        logger.info("回车")
        pyautogui.press("enter")

    @staticmethod
    def screenshot(file_name=None, position: str = None):
        return screenshot_util(pyautogui, file_name=file_name, position=position)


if __name__ == '__main__':
    MacDriver(pkg_name='111').get_location()






import time
from typing import Union

from kuto.utils.common import draw_red_by_rect
from kuto.utils.log import logger
from kuto.utils.exceptions import KError

from kuto.core.android.driver import AndroidDriver
from kuto.core.android.element import AdrElem
from kuto.core.ios.driver import IosDriver
from kuto.core.ios.element import IosElem
from kuto.core.image.element import ImageElem
from kuto.core.ocr.element import OCRElem
from kuto.core.web.driver import PlayWrightDriver as WebDriver
from kuto.core.web.element import WebElem, FraElem
from kuto.core.mac.driver import MacDriver
from kuto.core.mac.element import MacElem
from kuto.core.api.request import HttpReq
from requests_toolbelt import MultipartEncoder


class CoorElem(object):
    """根据坐标进行定位"""

    def __init__(self,
                 driver: Union[AndroidDriver, IosDriver, MacDriver] = None,
                 coor: tuple = None,
                 desc: str = None,
                 debug: bool = False):
        self.driver = driver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc
        self._coor = coor
        self._debug = debug

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def find(self, timeout=3):
        time.sleep(timeout)
        x, y = self._coor
        if self._debug is True:
            file_path = self.driver.screenshot(f'坐标定位结果-{self._desc}')
            draw_red_by_rect(file_path, (int(x) - 100, int(y) - 100, 200, 200))
        return x, y

    def click(self, timeout=3):
        logger.info(f"点击 {self._desc}")
        x, y = self.find(timeout=timeout)
        self.driver.click(x, y)

    def input(self, text: str, timeout=3):
        logger.info(f"点击 {self._desc}， 输入 {text}")
        x, y = self.find(timeout=timeout)
        self.driver.click(x, y)
        self.driver.input(text)


__all__ = [
    "AndroidDriver",
    "AdrElem",
    "IosDriver",
    "IosElem",
    "ImageElem",
    "OCRElem",
    "WebDriver",
    "WebElem",
    "MacDriver",
    "MacElem",
    "CoorElem",
    "FraElem",
    "HttpReq",
    "MultipartEncoder"
]




from kuto.core import *
from kuto.case import Case, Page
from kuto.running.runner import main
from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.pytest_util import depend, order, \
    data, file_data
from allure import *


__version__ = "0.0.62"
__description__ = "移动、web、接口自动化测试框架"



# @Time    : 2022/2/11 16:52
# @Author  : kang.yang@qizhidao.com
# @File    : mysql_util.py
# pip install PyMySQL==0.10.1
import pymysql


class MysqlDB(object):
    def __init__(self, host, username, password, database):
        self.host = host
        self.username = username
        self.password = password
        self.database = database
        self.con = pymysql.connect(host=self.host,
                                   user=self.username,
                                   password=self.password,
                                   database=self.database,
                                   charset='utf8',
                                   )
        # cursorclass=pymysql.cursors.DictCursor
        # 该配置详会导致fetchone方法返回的是标题
        self.cursor = self.con.cursor()

    def insert(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'插入失败: {e}')
            self.con.rollback()
        else:
            print('插入成功')
        finally:
            self.close()

    def delete(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'删除失败: {e}')
            self.con.rollback()
        else:
            print('删除成功')
        finally:
            self.close()

    def update(self, sql):
        try:
            self.cursor.execute(sql)
            self.con.commit()
        except Exception as e:
            print(f'更新失败: {e}')
            self.con.rollback()
        else:
            print('更新成功')
        finally:
            self.close()

    def select(self, sql):
        try:
            self.cursor.execute(sql)
        except Exception as e:
            print(f'查询失败: {e}')
        else:
            print('查询成功,', end=' ')
            items = list(self.cursor.fetchall())
            print(f'共查询出: {len(items)} 行数据')
            return items
        finally:
            self.close()

    def close(self):
        self.cursor.close()
        self.con.close()











"""
@Author: kang.yang
@Date: 2023/5/12 20:49
"""
import time

from playwright.sync_api import sync_playwright, expect

from kuto.utils.common import screenshot_util
from kuto.utils.log import logger


class PlayWrightDriver:

    def __init__(self, browserName: str, headless: bool = False, state: dict = None):
        logger.info("初始化playwright驱动")
        self.browserName = browserName
        self.headless = headless

        self.playwright = sync_playwright().start()
        if browserName == 'firefox':
            self.browser = self.playwright.firefox.launch(headless=headless)
        elif browserName == 'webkit':
            self.browser = self.playwright.webkit.launch(headless=headless)
        else:
            self.browser = self.playwright.chromium.launch(headless=headless)
        if state:
            self.context = self.browser.new_context(storage_state=state, no_viewport=True)
        else:
            self.context = self.browser.new_context(no_viewport=True)
        self.page = self.context.new_page()

    def switch_tab(self, locator):
        logger.info("开始切换tab")
        with self.page.expect_popup() as popup_info:
            locator.click()
        self.page = popup_info.value

    def open_url(self, url):
        logger.info(f"访问页面: {url}")
        self.page.goto(url)

    def storage_state(self, path=None):
        logger.info("保存浏览器状态信息")
        if not path:
            raise ValueError("路径不能为空")
        self.context.storage_state(path=path)

    @property
    def page_content(self):
        """获取页面内容"""
        logger.info("获取页面内容")
        content = self.page.content()
        logger.info(content)
        return content

    def set_cookies(self, cookies: list):
        logger.info("添加cookie并刷新页面")
        self.context.add_cookies(cookies)
        self.page.reload()

    def screenshot(self, file_name=None, position=None):
        return screenshot_util(self.page, file_name=file_name, position=position)

    def close(self):
        logger.info("关闭浏览器")
        self.page.close()
        self.context.close()
        self.browser.close()
        self.playwright.stop()

    def assert_title(self, title: str, timeout: int = 5):
        logger.info(f"断言页面标题等于: {title}")
        expect(self.page).to_have_title(title, timeout=timeout * 1000)

    def assert_url(self, url: str, timeout: int = 5):
        logger.info(f"断言页面url等于: {url}")
        expect(self.page).to_have_url(url, timeout=timeout * 1000)


if __name__ == '__main__':
    driver1 = PlayWrightDriver(browserName="chrome")
    driver1.open_url("https://patents.qizhidao.com/search/simple-result?from=simple&searchBtntype=searchBtn&businessSource=PC%E6%9F%A5%E4%B8%93%E5%88%A9&statement=%E4%BC%81%E7%9F%A5%E9%81%93")
    time.sleep(5)
    print(driver1.page.url)




"""
@Author: kang.yang
@Date: 2023/9/14 19:34
"""
import time

from kuto.core.mac.driver import MacDriver
from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.utils.common import draw_red_by_rect


class MacElem(object):
    """mac端元素定位，图像建议使用shift+command+4进行截取"""

    def __init__(self,
                 driver: MacDriver = None,
                 image: str = None,
                 desc: str = None,
                 debug: bool = False):
        self.driver = driver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc
        self.target_image = image
        self._debug = debug

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def find(self, timout=3):
        logger.info(f'查找元素: {self._desc}')
        for i in range(3):
            logger.info(f'第{i + 1}次查找:')
            point = self.driver.locate(self.target_image)
            if point is not None:
                x, y = point
                logger.info("查找成功")
                if self._debug is True:
                    file_path = self.driver.screenshot(f'{self._desc}_查找成功')
                    draw_red_by_rect(file_path, (int(x * 2) - 50, int(y * 2) - 50, 100, 100))
                return x, y
            else:
                logger.info("查找失败")
                time.sleep(timout)
        else:
            self.driver.screenshot(f'{self._desc}_查找失败')
            raise KError(f'{self._desc}_查找失败')

    def exists(self, timeout=3):
        logger.info(f"检查 {self._desc} 是否存在")
        result = False
        try:
            _element = self.find(timout=timeout)
            result = True
        except:
            result = False
        finally:
            logger.info(result)
            return result

    def click(self):
        logger.info(f"点击 {self._desc}")
        x, y = self.find()
        self.driver.click(x, y)

    def input(self, text: str):
        logger.info(f"输入 {text}")
        self.click()
        self.driver.input(text)


if __name__ == '__main__':
    pass




"""
@Author: kang.yang
@Date: 2023/8/2 20:23
"""
from kuto import Page, WebElem as Elem

"""
定位方式：优先使用css，大规模开展自动化前需要开发给相关元素加上唯一标识的测试id
css: 根据cssSelector进行定位，https://zhuanlan.zhihu.com/p/571510714
xpath: 根据xpath进行定位，教程：https://zhuanlan.zhihu.com/p/571060826
text: 根据元素的可视文本内容进行定位
holder: 根据输入框placeHolder进行定位
index: 获取第index个定位到的元素
"""


class IndexPage(Page):
    """首页"""
    loginBtn = Elem(text='登录/注册', desc='登录/注册按钮')
    patentText = Elem(text='查专利', desc='查专利文本')


class LoginPage(Page):
    """登录页"""
    pwdLoginTab = Elem(text='帐号密码登录', desc='账号密码登录tab')
    userInput = Elem(holder='请输入手机号码', desc='账号输入框')
    pwdInput = Elem(holder='请输入密码', desc='密码输入框')
    licenseBtn = Elem(css="span.el-checkbox__inner", index=1, desc='协议选择按钮')
    loginBtn = Elem(text='立即登录', desc='立即登录按钮')
    firstCompanyIcon = Elem(xpath="(//img[@class='right-icon'])[1]", desc='第一家公司')






"""
@Author: kang.yang
@Date: 2023/5/16 14:37
"""
import kuto

from pub import Pub


class TestWebDemo(kuto.Case):

    def start(self):
        self.pub = Pub(self.driver)

    @kuto.title("登录")
    def test_login(self):
        self.open_url()
        self.pub.pwd_login()
        self.assert_url()
        self.screenshot("首页")


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform="web",
        brow="chrome",
        host="https://www-test.qizhidao.com"
    )




import typing

from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector

from kuto.core.android.driver import AndroidDriver
from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.utils.common import calculate_time, draw_red_by_coordinate


class AdrElem(object):
    """
    安卓元素定义
    """

    def __init__(self,
                 AdrDriver: AndroidDriver = None,
                 desc: str = None,
                 rid: str = None,
                 className: str = None,
                 text: str = None,
                 textCont: str = None,
                 xpath: str = None,
                 index: int = None,
                 debug: bool = False):
        """
        @param AdrDriver: 安卓驱动，必填
        @param desc: 元素描述，必填
        @param rid: resourceId定位
        @param className: className定位
        @param text: text定位
        @param textCont: text包含
        @param xpath: xpath定位
        @param index: 定位出多个元素时，指定索引，从0开始
        @param debug: 是否调试
        """
        self._driver = AdrDriver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc

        self._kwargs = {}
        if rid is not None:
            self._kwargs["resourceId"] = rid
        if className is not None:
            self._kwargs["className"] = className
        if text is not None:
            self._kwargs["text"] = text
        if textCont is not None:
            self._kwargs["textContains"] = textCont
        if xpath:
            self._kwargs["xpath"] = xpath
        if index is not None:
            self._kwargs["instance"] = index

        self._xpath = xpath
        self._debug = debug

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    @calculate_time
    def find(self, timeout=10, watch=None):
        """
        增加截图的方法
        @param timeout: 每次查找时间
        @param watch: 增加弹窗检测，定位方式列表，用text定位
        watch为True时，使用内置库
            when("继续使用")
            when("移入管控").when("取消")
            when("^立即(下载|更新)").when("取消")
            when("同意")
            when("^(好的|确定)")
            when("继续安装")
            when("安装")
            when("Agree")
            when("ALLOW")
        watch为list时，使用内置库+watch
        @return:
        """
        rid = self._kwargs.get("resourceId", None)
        if rid is not None:
            pkg_name = self._driver.pkg_name
            if pkg_name not in rid:
                if "id/" not in rid:
                    self._kwargs["resourceId"] = pkg_name + ":id/" + rid
                else:
                    self._kwargs["resourceId"] = pkg_name + ":" + rid

        def _find():
            if self._xpath is not None:
                logger.info(f'查找控件: xpath={self._xpath}')
            else:
                logger.info(f'查找控件: {self._kwargs}')
            _element = self._driver.d.xpath(self._xpath) if \
                self._xpath is not None else self._driver.d(**self._kwargs)

            if _element.wait(timeout=timeout):
                logger.info(f"查找成功")
                if self._debug is True:
                    file_path = self._driver.screenshot(self._desc + "_查找成功")
                    logger.debug(file_path)
                    draw_red_by_coordinate(file_path, _element.bounds())
                return _element
            else:
                logger.info(f"查找失败")
                self._driver.screenshot(self._desc + "_查找失败")
                raise KError(f"控件 {self._kwargs} 查找失败")

        if watch:
            logger.info("开启弹窗检测")
            if isinstance(watch, list):
                with self._driver.d.watch_context(builtin=True) as ctx:
                    for text in watch:
                        ctx.when(text).click()
                    ctx.wait_stable()
                    logger.info("结束检测")
                    return _find()
            else:
                with self._driver.d.watch_context(builtin=True) as ctx:
                    ctx.wait_stable()
                    logger.info("结束检测")
                    return _find()
        else:
            return _find()

    @property
    def text(self):
        logger.info(f"获取 {self._desc} 文本属性")
        _elem = self.find(timeout=3)
        if isinstance(_elem, XPathSelector):
            elems = _elem.all()
        else:
            elems = list(_elem)
        text = []
        for elem in elems:
            text.append(elem.get_text())
        logger.info(text)
        return text

    def exists(self, timeout=3):
        logger.info(f"检查 {self._desc} 是否存在")
        result = False
        try:
            _element = self.find(timout=timeout)
            result = True
        except:
            result = False
        finally:
            logger.info(result)
            return result

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector],
                      offset=(0.5, 0.5)):
        """
        修正控件中心坐标
        """
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    def click(self, timeout=5, watch=None):
        logger.info(f"点击 {self._desc}")
        element = self.find(timeout=timeout, watch=watch)
        # 这种方式经常点击不成功，感觉是页面刷新有影响
        # element.click()
        x, y = self._adapt_center(element)
        self._driver.d.click(x, y)

    def click_exists(self, timeout=3, watch=None):
        logger.info(f"{self._desc} 存在才点击")
        if self.exists(timeout=timeout):
            self.click(watch=watch)

    def input(self, text, watch=None):
        logger.info(f"输入文本: {text}")
        self.find(watch=watch).set_text(text)

    def input_exists(self, text: str, timeout=3, watch=None):
        logger.info(f"{self._desc} 存在才输入: {text}")
        if self.exists(timeout=timeout):
            self.input(text, watch=watch)

    def input_pwd(self, text, watch=None):
        """密码输入框输入有时候用input输入不了"""
        logger.info(f"输入密码: {text}")
        self.find(watch=watch).click()
        self._driver.d(focused=True).set_text(text)

    def clear(self, watch=None):
        logger.info("清空输入框")
        self.find(watch=watch).clear_text()

    def assert_exists(self, timeout=3):
        logger.info(f"断言 {self._desc} 存在")
        status = self.exists(timeout=timeout)
        assert status, "控件不存在"

    def assert_text(self, text, timeout=3, watch=None):
        logger.info(f"断言 {self._desc} 文本属性包括: {text}")
        self.find(timeout=timeout, watch=watch)
        _text = self.text
        assert text in _text, f"文本属性 {_text} 不包含 {text}"


if __name__ == '__main__':
    driver = AndroidDriver(
        device_id="UJK0220521066836",
        pkg_name="com.qizhidao.clientapp"
    )
    driver.start_app()
    AdrElem(driver, rid='com.qizhidao.clientapp:id/bottom_btn').click()
    AdrElem(driver, text='我的', debug=True).click()









"""
@Author: kang.yang
@Date: 2023/9/20 10:54
"""
import kuto

from page.win_page import WinPage


class TestMacDemo(kuto.Case):
    """Mac应用demo"""

    def start(self):
        self.win_page = WinPage(self.driver)

    def test_1(self):
        self.win_page.num_3.click()
        self.win_page.x.click()
        self.win_page.num_5.click()
        self.win_page.equal.click()
        self.driver.screenshot("计算结果")


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform='win',
        pkg_name='calc.exe',
    )



import time
import threading

from kuto.utils.log import logger
from kuto.core.ios.driver import IosDriver
from kuto.utils.common import calculate_time, draw_red_by_rect
from kuto.utils.exceptions import KError


lock = threading.Lock()


class IosElem(object):
    """
    IOS原生元素定义
    """

    def __init__(self,
                 iosDriver: IosDriver = None,
                 desc: str = None,
                 name: str = None,
                 label: str = None,
                 labelCont: str = None,
                 value: str = None,
                 valueCont: str = None,
                 text: str = None,
                 textCont: str = None,
                 className: str = None,
                 xpath: str = None,
                 index: int = None,
                 debug: bool = False):
        """
        param iosDriver,
        param desc,
        param name,
        param label,
        param value,
        param text,
        param className,
        param xpath,
        param index: 索引
        @param debug: 是否调试
        """
        self._driver = iosDriver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc

        self._kwargs = {}
        if name is not None:
            self._kwargs["name"] = name
        if label is not None:
            self._kwargs["label"] = label
        if labelCont is not None:
            self._kwargs["labelContains"] = labelCont
        if value is not None:
            self._kwargs["value"] = value
        if valueCont is not None:
            self._kwargs["valueContains"] = valueCont
        if text is not None:
            self._kwargs["text"] = text
        if textCont is not None:
            self._kwargs["textContains"] = textCont
        if className is not None:
            self._kwargs["className"] = className
        if index is not None:
            self._kwargs["index"] = index

        self._xpath = xpath
        self._debug = debug

    def __get__(self, instance, owner):
        """po模式中element初始化不需要带driver的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    def _match(self, text):
        """
        判断元素是否存在当前页面
        @return:
        """
        _element = self._driver.d(text=text)
        result = text if _element.wait(timeout=0.1, raise_error=False) else None
        return result

    def _watch(self, loc_list, timeout=3):
        logger.info("开始弹窗检测")
        # 多线程match，如果match到，获取第一个非None内容，进行点击
        # match完休息1s，如果休息3s也没有match到，就停止（定义一个flag，match到就清零）
        # 如果3s内仍然能match到就继续（如果flag大于3就停止）
        _build_info = ["允许", "使用App时允许", "始终允许", "同意"]
        if loc_list is True:
            loc_list = _build_info
        else:
            loc_list = list(set(_build_info + loc_list))
        flag = timeout
        while flag > 0:
            import concurrent.futures

            logger.info(f"匹配: {loc_list}")
            with concurrent.futures.ThreadPoolExecutor() as executor:
                results = executor.map(self._match, loc_list)
                results = [item for item in results if item is not None]

            if results:
                logger.info(f"匹配到: {results}")
                self._driver.d(text=results[0]).click()
                logger.info("点击成功")
                flag = timeout
            else:
                logger.info("匹配失败")

            flag -= 1
            time.sleep(1)
        logger.info("结束检测")

    @calculate_time
    def find(self, timeout=10, watch=None):
        """
        针对元素定位失败的情况，抛出KError异常
        @param timeout:
        @param watch：弹窗检测
        ["允许", "同意"]
        @return:
        """
        def _find():
            if self._xpath is not None:
                logger.info(f'查找控件: xpath={self._xpath}')
            else:
                logger.info(f'查找控件: {self._kwargs}')

            _element = self._driver.d.xpath(self._xpath) if \
                self._xpath else self._driver.d(**self._kwargs)
            try:
                if _element.wait(timeout=timeout):
                    logger.info(f"查找成功")
                    if self._debug is True:
                        file_path = self._driver.screenshot(self._desc + "_查找成功")
                        rect = _element.bounds
                        scale = self._driver.d.scale
                        rect_tuple = (rect.x * scale, rect.y * scale, rect.width * scale, rect.height * scale)
                        logger.debug(rect_tuple)
                        draw_red_by_rect(file_path, rect_tuple)
                    return _element
                else:
                    logger.info(f"查找失败")
                    self._driver.screenshot(self._desc + "_查找失败")
                    raise KError(f"控件 {self._kwargs} 查找失败")
            except ConnectionError:
                logger.info('wda连接失败, 进行重连!!!')
                # 由于WDA会意外链接错误
                self._driver = IosDriver(self._driver.device_id, self._driver.pkg_name)
                time.sleep(5)

                logger.info('重连成功, 重新开始查找控件')
                if _element.wait(timeout=timeout):
                    logger.info(f"查找成功")
                    return _element
                else:
                    logger.info(f"查找失败")
                    self._driver.screenshot(self._desc + "_查找失败")
                    raise KError(f"控件 {self._kwargs} 查找失败")

        if watch:
            self._watch(watch)
            return _find()
        else:
            return _find()

    @property
    def text(self):
        """获取元素文本"""
        logger.info(f"获取 {self._desc} 文本属性")
        text = [elem.text for elem in self.find(timeout=5).find_elements()]
        logger.info(text)
        return text

    def exists(self, timeout=3):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f"检查 {self._desc} 是否存在")
        result = False
        try:
            _element = self.find(timout=timeout)
            result = True
        except:
            result = False
        finally:
            logger.info(result)
            return result

    def _adapt_center(self, timeout=5, watch=None):
        """
        修正控件中心坐标
        """
        bounds = self.find(timeout=timeout, watch=watch).bounds
        left_top_x, left_top_y, width, height = \
            bounds.x, bounds.y, bounds.width, bounds.height
        center_x = int(left_top_x + width/2)
        center_y = int(left_top_y + height/2)
        logger.info(f'{center_x}, {center_y}')
        return center_x, center_y

    def click(self, timeout=5, watch=None):
        """
        单击
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        """
        logger.info(f'点击 {self._desc}')
        x, y = self._adapt_center(timeout=timeout, watch=watch)
        self._driver.d.appium_settings({"snapshotMaxDepth": 0})
        self._driver.d.tap(x, y)
        self._driver.d.appium_settings({"snapshotMaxDepth": 50})

    def click_exists(self, timeout=3):
        logger.info(f"{self._desc} 存在才点击")
        if self.exists(timeout=timeout):
            self.click()

    def clear(self):
        """清除文本"""
        logger.info("清除输入框文本")
        self.find().clear_text()

    def input(self, text):
        """输入内容"""
        logger.info(f"输入文本：{text}")
        self.find().set_text(text)

    def input_exists(self, text: str, timeout=3):
        logger.info(f"{self._desc} 存在才输入: {text}")
        if self.exists(timeout=timeout):
            self.input(text)
            logger.info("输入成功")
        else:
            logger.info("控件不存在")

    def assert_exists(self, timeout=3):
        logger.info(f"断言 {self._desc} 存在")
        status = self.exists(timeout=timeout)
        assert status, f"控件不存在"

    def assert_text(self, text, timeout=3):
        logger.info(f"断言 {self._desc} 文本属性包括: {text}")
        self.find(timeout=timeout)
        _text = self.text
        assert text in _text, f"文本属性 {_text} 不包含 {text}"


if __name__ == '__main__':
    driver = IosDriver(
        device_id="00008101-000E646A3C29003A",
        pkg_name="com.qizhidao.company"
    )
    driver.start_app()
    IosElem(driver, text='我的', debug=True).click()



















"""
@Author: kang.yang
@Date: 2023/8/24 15:56
"""
from page.web_page import IndexPage, LoginPage


class Pub(object):
    """公共方法"""

    def __init__(self, driver):
        self.driver = driver
        self.index_page = IndexPage(self.driver)
        self.login_page = LoginPage(self.driver)

    def pwd_login(self, username="13652435335", password="wz123456@QZD"):
        """账号密码登录流程"""
        self.index_page.loginBtn.click()
        self.login_page.pwdLoginTab.click()
        self.login_page.userInput.input(username)
        self.login_page.pwdInput.input(password)
        self.login_page.licenseBtn.click()
        self.login_page.loginBtn.click()
        self.login_page.firstCompanyIcon.click()



# @Time    : 2022/2/22 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : request.py
import json as json_util
import logging
import re
from functools import wraps

import jmespath
from jsonschema import validate, ValidationError
from requests.packages import urllib3
from urllib import parse

import requests

from kuto.utils.config import config
from kuto.utils.log import logger

# 去掉requests本身的日志
urllib3_logger = logging.getLogger("urllib3")
urllib3_logger.setLevel(logging.CRITICAL)

# 去掉不设置证书的报警
urllib3.disable_warnings()


def formatting(msg):
    """formatted message"""
    if isinstance(msg, dict):
        return json_util.dumps(msg, indent=2, ensure_ascii=False)
    return msg


def request(func):

    @wraps(func)
    def wrapper(*args, **kwargs):
        logger.info("-------------- Request -----------------[🚀]")
        # 给接口带上默认域名
        # 从配置文件中读取域名
        host = config.get_api("base_url")
        # 如果接口路径不以http开头，把域名写到key为url的位置参数中或者第一个参数中
        if "url" in kwargs:
            path: str = kwargs.get("url", "")
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                kwargs["url"] = url
            else:
                url = path
        else:
            path = list(args)[1]
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                args_list = list(args)
                args_list[1] = url
                args = tuple(args_list)
            else:
                url = path

        # 请求头处理，写入登录态
        # 从配置文件获取登录用户和游客的请求头
        if kwargs.get("login", True):
            login_header: dict = config.get_api("login")
        else:
            login_header: dict = config.get_api("visit")
        # 把用例脚本中设置的请求头加进来
        header_user_set = kwargs.pop("headers", {})
        login_header.update(header_user_set)
        # 把组装好的请求头装回到kwargs当中
        kwargs["headers"] = login_header

        # 更新超时时间
        timeout_user_set = kwargs.pop("timeout", None)  # 用例脚本中设置的超时时间
        kwargs["timeout"] = timeout_user_set if timeout_user_set else 10

        # 发送请求
        r = func(*args, **kwargs)

        # 输出请求参数日志
        logger.debug("[method]: {m}      [url]: {u}".format(m=func.__name__.upper(), u=url))
        auth = kwargs.get("auth", "")
        if auth:
            logger.debug(f"[auth]:\n {formatting(auth)}")
        logger.debug(f"[headers]:\n {formatting(dict(r.request.headers))}")
        cookies = kwargs.get("cookies", "")
        if cookies:
            logger.debug(f"[cookies]:\n {formatting(cookies)}")
        params = kwargs.get("params", "")
        if params:
            logger.debug(f"[params]:\n {formatting(params)}")
        data = kwargs.get("static", "")
        if data:
            logger.debug(f"[static]:\n {formatting(data)}")
        json = kwargs.get("json", "")
        if json:
            logger.debug(f"[json]:\n {formatting(json)}")

        # 保存响应结果并输出日志
        status_code = r.status_code
        headers = r.headers
        content_type = headers.get("Content-Type")
        ResponseResult.status_code = status_code
        logger.info("-------------- Response ----------------")
        logger.debug(f"[status]: {status_code}")
        logger.debug(f"[headers]: {formatting(headers)}")
        try:
            resp = r.json()
            logger.debug(f"[type]: json")
            logger.debug(f"[response]:\n {formatting(resp)}")
            ResponseResult.response = resp
        except Exception:
            # 非json响应数据，根据响应内容类型进行判断
            logger.info("response is not json type static.")
            if content_type is not None:
                if "text" not in content_type:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.content}")
                    ResponseResult.response = r.content
                else:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.text}")
                    ResponseResult.response = r.text
            else:
                logger.debug('ContentType为空，响应异常！！！')
                ResponseResult.response = r.text

    return wrapper


class ResponseResult:
    # 并发执行不会串数据，是因为我用的是多进程而不是多线程吧???
    status_code = 200
    response = None


class HttpReq(object):
    @request
    def get(self, url, params=None, verify=False, login=True, **kwargs):
        return requests.get(url, params=params, verify=verify, **kwargs)

    @request
    def post(self, url, data=None, json=None, verify=False, login=True, **kwargs):
        return requests.post(url, data=data, json=json, verify=verify, **kwargs)

    @request
    def put(self, url, data=None, json=None, verify=False, login=True, **kwargs):
        if json is not None:
            data = json_util.dumps(json)
        return requests.put(url, data=data, verify=verify, **kwargs)

    @request
    def delete(self, url, verify=False, login=True, **kwargs):
        return requests.delete(url, verify=verify, **kwargs)

    @property
    def response(self):
        """
        Returns the result of the response
        :return: response
        """
        return ResponseResult.response

    # 断言
    @staticmethod
    def assert_status(status_code):
        """
        状态码
        """
        actual_code = ResponseResult.status_code
        logger.info(f"断言: {actual_code} 等于 {status_code}")
        assert (
                actual_code == status_code
        ), f"{ResponseResult} != {status_code}"

    @staticmethod
    def assert_schema(schema, response=None) -> None:
        """
        Assert JSON Schema
        doc: https://json-schema.org/
        """
        logger.info(f"assertSchema -> {formatting(schema)}.")

        if response is None:
            response = ResponseResult.response

        try:
            validate(instance=response, schema=schema)
        except ValidationError as msg:
            assert "Response static" == "Schema static", msg

    @staticmethod
    def assert_eq(path, value):
        """
        等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 等于 {value}")
        assert search_value == value, f"{search_value} != {value}"

    @staticmethod
    def assert_neq(path, value):
        """
        不等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不等于 {value}")
        assert search_value != value, f"{search_value} == {value}"

    @staticmethod
    def assert_leq(path, value):
        """
        长度等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 等于 {value}")
        assert len(search_value) == value, f"{len(search_value)} != {value}"

    @staticmethod
    def assert_lgt(path, value):
        """
        长度大于等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 大于等于 {value}")
        assert len(search_value) >= value, f"{len(search_value)} < {value}"

    @staticmethod
    def assert_llt(path, value):
        """
        长度小于等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 小于等于 {value}")
        assert len(search_value) <= value, f"{len(search_value)} > {value}"

    @staticmethod
    def assert_gt(path, value):
        """
        大于等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 大于等于 {value}")
        assert search_value >= value, f"{search_value} < {value}"

    @staticmethod
    def assert_lt(path, value):
        """
        小于等于
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 小于等于 {value}")
        assert search_value <= value, f"{search_value} 大于 {value}"

    @staticmethod
    def assert_rg(path, start, end):
        """
        范围
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 在 [{start}, {end}] 范围内")
        assert (search_value >= start) & (
                search_value <= end
        ), f"{search_value} 不在[{start}, {end}]范围内"

    @staticmethod
    def assert_in(path, value):
        """
        被包含
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 被 {value} 包含")
        assert search_value in value, f"{value} 不包含 {search_value}"

    @staticmethod
    def assert_nin(path, value):
        """
        不被包含
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不被 {value} 包含")
        assert search_value not in value, f"{value} 包含 {search_value}"

    @staticmethod
    def assert_ct(path, value):
        """
        包含
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 包含 {value}")
        assert value in search_value, f"{search_value} 不包含 {value}"

    @staticmethod
    def assert_nct(path, value):
        """
        不包含
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不包含 {value}")
        assert value not in search_value, f"{search_value} 包含 {value}"

    @staticmethod
    def assert_type(path, value_type):
        """
        字段类型
        """
        if not isinstance(value_type, type):
            if value_type == "int":
                value_type = int
            elif value_type == "str":
                value_type = str
            elif value_type == "list":
                value_type = list
            elif value_type == "dict":
                value_type = dict
            else:
                value_type = str

        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 是 {value_type} 类型")
        assert isinstance(
            search_value, value_type
        ), f"{search_value} 不是 {value_type} 类型"

    @staticmethod
    def assert_start(path, value):
        """
        以什么开始
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 以 {value} 开头")
        assert search_value.startswith(value), f"{search_value} 不以 {value} 开头"

    @staticmethod
    def assert_end(path, value):
        """
        以什么结束
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 以 {value} 结尾")
        assert search_value.endswith(value), f"{search_value} 不以 {value} 结尾"

    @staticmethod
    def assert_match(path, value):
        """正则匹配"""
        search_value = jmespath.search(path, ResponseResult.response)
        match_obj = re.match(r"" + value, search_value, flags=re.I)
        logger.info(f"断言: {search_value} 匹配正则表达式 {value} 成功")
        assert match_obj is not None, f"结果 {search_value} 匹配失败"





"""
@Author: kang.yang
@Date: 2023/7/31 18:14
"""
from kuto import AndroidDriver, AdrElem

driver = AndroidDriver(device_id='UJK0220521066836',
                       pkg_name='com.qizhidao.clientapp')
driver.start_app()
AdrElem(driver, rid='com.qizhidao.clientapp:id/btn_login').click()
AdrElem(driver, text='其他手机号码登录').click()
AdrElem(driver, text='帐号密码登录').click()
AdrElem(driver, rid='com.qizhidao.clientapp:id/phone_et').input_exists("13652435335")
AdrElem(driver, rid='com.qizhidao.clientapp:id/clear_edit_text').input_pwd("wz123456@QZD")
AdrElem(driver, rid='com.qizhidao.clientapp:id/pwd_check_box_layout').click_exists()
AdrElem(driver, text='登录').click()
AdrElem(driver, rid='com.qizhidao.clientapp:id/common_list_rv').click_exists()
AdrElem(driver, rid='com.qizhidao.clientapp:id/skip_btn').click_exists()
driver.assert_act('.main.HomeActivity')
driver.stop_app()






"""
使用allure需要下载，并配置到环境变量，下载地址：https://github.com/allure-framework/allure2/releases
"""
import pytest
import json
import yaml

from kuto.utils.excel import Excel, CSV


def order(index):
    """
    指定用例执行顺序, pip install pytest-ordering==0.6
    doc: https://blog.csdn.net/weixin_43880991/article/details/116221362
    """
    return pytest.mark.run(order=index)


def depend(depends: list or str = None, name=None):
    """
    设置用例依赖关系, pip install pytest-dependency==0.5.1
    doc: https://www.cnblogs.com/se7enjean/p/13513131.html
    """
    if isinstance(depends, str):
        depends = [depends]
    return pytest.mark.dependency(name=name, depends=depends)


# 参数化数据
def data(list_data: list):
    """
    必须传入一个list，使用时通过在参数列表传入parma进行调用
    """
    return pytest.mark.parametrize('param', list_data)


# 从json文件获取数据进行参数化
def file_data(file=None, key=None, row=None):
    # logger.debug(config.get_env())

    """
    @param row: 第几行，针对csv和excel文件
    @param file: 文件名
    @param key: 针对json和yaml文件
    """
    # 获取被装饰方法的目录
    # stack_t = sys_inspect.stack()
    # ins = sys_inspect.getframeinfo(stack_t[1][0])
    # file_dir = os.path.dirname(os.path.abspath(ins.filename))
    # # logger.debug(file_dir)
    # parent_dir = os.path.dirname(file_dir)
    # parent_dir_dir = os.path.dirname(parent_dir)
    # parent_dir_dir_dir = os.path.dirname(parent_dir_dir)
    #
    # # 判断当前、父目录、爷目录、太爷爷目录下的test_data目录下是否有file_name文件
    # path_list = [file_dir, parent_dir, parent_dir_dir, parent_dir_dir_dir]
    # file_path = None
    # full_list = []
    # for _path in path_list:
    #     # environ = config.get_env()
    #     # if environ:
    #     #     full_path = os.path.join(_path, 'static', environ, static)
    #     # else:
    #     full_path = os.path.join(_path, 'static', static)
    #     full_list.append(full_path)
    #     # print(full_path)
    #     if os.path.isfile(full_path) is True:
    #         file_path = full_path
    #         break
    # if file_path is None:
    #     raise Exception(f"can not found {static} in {full_list}")
    file_path = file  # 去掉文件查找机制，不好理解，用处也不大

    # logger.debug(file_path)
    if file_path.endswith(".json"):
        content = read_json(file_path, key)
    elif file_path.endswith(".yml"):
        content = read_yaml(file_path, key)
    elif file_path.endswith(".csv"):
        content = read_csv(file_path, row)
    elif file_path.endswith(".xlsx"):
        content = read_excel(file_path, row)
    else:
        raise TypeError("不支持的文件类型，仅支持json、yml、csv、xlsx")

    if content:
        return data(content)
    else:
        raise ValueError('数据不能为空')


def read_json(file_path, key=None):
    """
    读取json文件中的指定key
    @return
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        json_data = json.load(f)
    if isinstance(json_data, list):
        return json_data
    else:
        if key:
            return json_data[key]
        raise ValueError('key不能为空')


def read_yaml(file_path, key=None):
    """
    读取yaml文件中的指定key
    @param file_path:
    @param key:
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        yaml_data = yaml.load(f, Loader=yaml.FullLoader)
    if isinstance(yaml_data, list):
        return yaml_data
    else:
        if key:
            return yaml_data[key]
        raise ValueError('key不能为空')


def read_csv(file_path, row=None):
    """
    读取csv文件中的指定行
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    csv_file = CSV(file_path)
    if row:
        csv_data = [csv_file.read_row_index(row)]
    else:
        csv_data = csv_file.read_all()
    return csv_data


def read_excel(file_path, row=None):
    """
    读取excel文件中的指定行，暂时只支持读取第一个sheet
    @param file_path: 文件名
    @param row: 行数，从1开始
    """
    excel_file = Excel(file_path)
    if row:
        excel_data = [excel_file.read_row_index(row)]
    else:
        excel_data = excel_file.read_all()
    return excel_data



import os
import re
import subprocess

import requests
import six

from kuto.utils.log import logger
from kuto.utils.exceptions import KError


class SibUtil:
    """sib操作
    本地需要部署sib环境，参考：https://github.com/SonicCloudOrg/sonic-ios-bridge
    """

    def __init__(self, host, port):
        self.host = host
        self.port = port

    @staticmethod
    def get_current_devices():
        """
        获取当前设备列表
        @return:
        """
        cmd = 'sib devices'
        output = os.popen(cmd).read()
        print(output)
        device_list = [item.split(' ')[0] for item in output.split('\n') if item]
        if device_list:
            return device_list
        else:
            raise KError(msg=f"无已连接设备")

    def connect(self):
        """
        连接远程设备
        @return:
        """
        cmd = f'sib remote connect --host {self.host} --port {self.port}'
        output = subprocess.getoutput(cmd)
        print(output)
        if 'succeeded' in output:
            logger.info('连接成功')
            return self.get_current_devices()[-1]
        else:
            raise KError(f'连接失败: \n{output}')

    def disconnect(self):
        """
        断开远程连接
        @return:
        """
        cmd = f'sib remote disconnect --host {self.host} --port {self.port}'
        logger.info(cmd)
        output = subprocess.getoutput(cmd)
        print(output)
        logger.info(output)

    @staticmethod
    def download_apk(src):
        """下载安装包"""
        if isinstance(src, six.string_types):
            if re.match(r"^https?://", src):
                logger.info(f'下载中: {src}')
                file_path = os.path.join(os.getcwd(), src.split('/')[-1])
                r = requests.get(src, stream=True)
                if r.status_code != 200:
                    raise IOError(
                        "Request URL {!r} status_code {}".format(src, r.status_code))
                with open(file_path, 'wb') as f:
                    f.write(r.content)
                logger.info(f'下载成功: {file_path}')
                return file_path
            elif os.path.isfile(src):
                return src
            else:
                raise IOError("static {!r} not found".format(src))

    @staticmethod
    def uninstall_app(device_id: str, pkg_name: str):
        """
        卸载应用
        @param device_id:
        @param pkg_name:
        @return:
        """
        # 使用sib命令安装应用
        cmd = f"sib app uninstall -u {device_id} -b {pkg_name}"
        logger.info(f"卸载应用: {pkg_name}")
        output = subprocess.getoutput(cmd)
        print(output)
        if "successful" in output.split()[-1]:
            logger.info(f"{device_id} 卸载应用 {pkg_name} 成功")
            return
        else:
            logger.info(f"{device_id} 卸载应用 {pkg_name} 失败，因为 {output}")

    def install_app(self, device_id: str, ipa_url: str):
        """
        安装app
        @return:
        """
        # 下载应用
        ipa_path = self.download_apk(ipa_url)

        # 使用sib命令安装应用
        cmd = f"sib app install -u {device_id} -p {ipa_path}"
        logger.info(f"安装应用: {ipa_url}")
        output = subprocess.getoutput(cmd)
        print(output)
        if "successful" in output.split()[-1]:
            logger.info(f"{device_id} 安装应用 {ipa_url} 成功")
            return
        else:
            logger.info(f"{device_id} 安装应用 {ipa_url} 失败，因为{output}")

        # 删除下载的安装包
        if 'http' in ipa_url:
            os.remove(ipa_path)


if __name__ == '__main__':
    sib = SibUtil('172.16.1.216', '51385')
    # device_id = sib.connect()
    pkg_name = 'com.qizhidao.company'
    # ipa_url = 'http://172.16.5.225:8081/AppUpdateTest/pre_qizhiyun_client_4.3.2.ipa'
    # sib.uninstall_app(device_id, pkg_name)
    # sib.disconnect()
    print(sib.get_current_devices())











import inspect
import json
import os
import pytest
import psutil

from kuto.utils.config import config, free_config
from kuto.utils.log import logger
from kuto.core.android.common import get_connected_adr_devices
from kuto.core.ios.common import get_connected_ios_devices


class TestMain(object):
    """
    Support for app、web、http
    """
    def __init__(self,
                 platform: str = None,
                 device_id=None,
                 pkg_name: str = None,
                 start: bool = True,
                 brow: str = None,
                 headless: bool = False,
                 path: str = None,
                 rerun: int = 0,
                 xdist: bool = False,
                 host: str = None,
                 headers: dict = None,
                 state: dict = None,
                 cookies: list = None
                 ):
        """
        @param platform: 测试平台，android、ios、web、api、mac、win
        @param device_id: 设备id，针对安卓和ios，可以是str和list，
        对安卓和ios来说也可以是远程服务
        @param pkg_name: 应用包名，针对安卓和ios
        @param start: 是否默认启动应用，针对安卓和ios
        @param brow: 浏览器类型，chrome、firefox、webkit
        @param path: 用例目录，None默认代表当前文件
        @param rerun: 失败重试次数
        @param xdist: 是否使用多进程执行
        @param host: 域名，针对接口和web
        @param headers: {
            "login": {},
            "visit": {}
        }
        @param state: 通过playwright的storage_state方法获取
        @param cookies:
        """

        # 公共参数保存
        config.set_common("platform", platform)
        # api参数保存
        config.set_api("base_url", host)
        if headers:
            if 'login' not in headers.keys():
                raise KeyError("without login key!!!")
            login_ = headers.pop('login', {})
            config.set_api('login', login_)
            visit_ = headers.pop('visit', {})
            config.set_api('visit', visit_)
        # app参数保存
        # 增加一段逻辑支持多进程以及设备调度
        # 把所有的设备id加入到空闲设备列表中（用文件保存）
        free_devices = []
        if device_id is None:
            # 获取当前连接的手机列表
            if platform == "android":
                free_devices = get_connected_adr_devices()
            elif platform == "ios":
                free_devices = get_connected_ios_devices()
            free_config.add_devices(free_devices)
        else:
            if isinstance(device_id, str):
                free_devices = [device_id]
                free_config.add_devices(free_devices)
            if isinstance(device_id, list):
                if xdist is True:
                    # 如果需要并发，才把所有设备放入空闲列表
                    free_devices = device_id
                    free_config.add_devices(free_devices)
                else:
                    # 如果不需要并发，把第一个设备放入空闲列表
                    free_devices = device_id[0:1]
                    free_config.add_devices(free_devices)
        config.set_app("pkg_name", pkg_name)
        config.set_app("auto_start", start)
        # web参数保存
        config.set_web("base_url", host)
        config.set_web("browser_name", brow)
        config.set_web("headless", headless)
        if state:
            config.set_web("state", json.dumps(state))
        if cookies:
            config.set_web("cookies", json.dumps(cookies))

        # 执行用例
        # logger.info('执行用例')
        if path is None:
            stack_t = inspect.stack()
            ins = inspect.getframeinfo(stack_t[1][0])
            file_dir = os.path.dirname(os.path.abspath(ins.filename))
            file_path = ins.filename
            if "\\" in file_path:
                this_file = file_path.split("\\")[-1]
            elif "/" in file_path:
                this_file = file_path.split("/")[-1]
            else:
                this_file = file_path
            path = os.path.join(file_dir, this_file)
        cmd_list = [
            '-sv',
            '--reruns', str(rerun),
            '--alluredir', 'report', '--clean-alluredir'
        ]
        if path:
            cmd_list.insert(0, path)
        if xdist:
            if platform in ["android", "ios"]:
                if len(free_devices) > 1:
                    # 设备数大于1才开启多进程
                    n = len(free_devices)
                    cpu_count = psutil.cpu_count()
                    if n > cpu_count:
                        n = cpu_count
                    cmd_list.insert(1, '-n')
                    cmd_list.insert(2, str(n))
            else:
                cmd_list.insert(1, '-n')
                cmd_list.insert(2, 'auto')
        logger.info(cmd_list)
        pytest.main(cmd_list)

        # 公共参数保存
        # api参数保存
        config.set_api("base_url", None)
        config.set_api('login', {})
        config.set_api('visit', {})
        # app参数保存
        # 增加一段逻辑支持多进程以及设备调度
        # 清空空闲设备列表
        free_config.clear_devices()
        config.set_app("device_id", None)
        config.set_app("pkg_name", None)
        config.set_app("auto_start", False)
        # config.set_app("errors", [])
        # web参数保存
        config.set_web("base_url", None)
        config.set_web("browser_name", "chrome")
        config.set_web("headless", False)
        config.set_web("state", None)
        config.set_web("cookies", None)


main = TestMain


if __name__ == '__main__':
    main()




"""
@Author: kang.yang
@Date: 2023/8/21 16:38
建议AirtestIDE截图，手机直接截图好像有问题
"""
import kuto
from page.image_page import ImagePage


class TestImageDemo(kuto.Case):
    """图像识别demo"""

    def start(self):
        self.image_page = ImagePage(self.driver)

    def test_nanshan_wtt(self):
        self.image_page.searchEntry.click()
        self.image_page.searchInput.input("南山文体通")
        self.image_page.searchResult.click()
        self.image_page.schoolEntry.click()
        self.sleep(5)


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform='android',
        device_id='UJK0220521066836',
        pkg_name='com.tencent.mm'
    )






import logging
import os
import time
from urllib import parse

# import allure
import allure
from selenium import webdriver
from selenium.common.exceptions import NoAlertPresentException
from selenium.webdriver import (
    ChromeOptions,
    DesiredCapabilities,
    EdgeOptions,
    FirefoxOptions,
)
from selenium.webdriver.chrome.service import Service as cService
from selenium.webdriver.edge.service import Service as eService
from selenium.webdriver.firefox.service import Service as fService
from selenium.webdriver.ie.service import Service as iService
from selenium.webdriver.remote.remote_connection import LOGGER
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager, IEDriverManager

from kuto.utils.common import screenshot_util
from kuto.utils.log import logger
from kuto.utils.webdriver_manager_extend import ChromeDriverManager


LOGGER.setLevel(logging.WARNING)


def get_selenium_driver(browserName: str, headless: bool = False, timeout: int = 60):
    browser = SeleniumBrowser(browserName, headless, timeout)
    return SeleniumDriver(browser)


class SeleniumBrowser(object):
    """
    根据关键词初始化浏览器操作句柄，
    如'chrome、google chrome、gc'代表chrome浏览器，
    如'firefox、ff'代表火狐浏览器，
    如'internet explorer、ie、IE'代表IE浏览器，
    如'edge'代表edge浏览器，
    如'safari'代表safari浏览器
    """

    def __init__(self, name, headless=False, timeout=60):
        self.name = name
        self.timeout = timeout
        self.headless = headless

    def start(self):
        if self.name == 'Chrome':
            return self.chrome()
        elif self.name == 'Firefox':
            return self.firefox()
        elif self.name == 'Webkit':
            return self.safari()
        else:
            return self.chrome()

    def chrome(self):
        chrome_options = ChromeOptions()
        chrome_options.add_argument(
            f"user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36"
        )
        if self.headless:
            chrome_options.add_argument("--headless")

        driver = webdriver.Chrome(
            options=chrome_options, executable_path=ChromeDriverManager().install()
        )

        driver.set_page_load_timeout(self.timeout)

        return driver

    def firefox(self):
        firefox_options = FirefoxOptions()
        firefox_options.add_argument(
            f"user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/112.0"
        )
        if self.headless:
            firefox_options.add_argument("--headless")

        driver = webdriver.Firefox(
            options=firefox_options, executable_path=GeckoDriverManager().install()
        )

        driver.set_page_load_timeout(self.timeout)

        return driver

    # @staticmethod
    # def ie():
    #     if IEConfig.command_executor == "":
    #         driver = webdriver.Ie(service=iService(IEDriverManager().install()))
    #     elif IEConfig.command_executor[:4] == "http":
    #         driver = webdriver.Remote(
    #             command_executor=IEConfig.command_executor,
    #             desired_capabilities=DesiredCapabilities.INTERNETEXPLORER.copy(),
    #         )
    #     else:
    #         driver = webdriver.Ie(executable_path=IEConfig.command_executor)
    #
    #     return driver
    #
    # @staticmethod
    # def edge():
    #     if EdgeConfig.options is None:
    #         edge_options = EdgeOptions()
    #     else:
    #         edge_options = EdgeConfig.options
    #
    #     if EdgeConfig.headless is True:
    #         edge_options.headless = True
    #
    #     if EdgeConfig.command_executor == "":
    #         driver = webdriver.Edge(
    #             options=edge_options,
    #             service=eService(EdgeChromiumDriverManager().install()),
    #         )
    #     elif EdgeConfig.command_executor[:4] == "http":
    #         driver = webdriver.Remote(
    #             options=edge_options,
    #             command_executor=EdgeConfig.command_executor,
    #             desired_capabilities=DesiredCapabilities.EDGE.copy(),
    #         )
    #     else:
    #         driver = webdriver.Edge(
    #             options=edge_options, executable_path=EdgeConfig.command_executor
    #         )
    #
    #     return driver

    def safari(self):
        # if (
        #     SafariConfig.command_executor != ""
        #     and SafariConfig.command_executor[:4] == "http"
        # ):
        #     return webdriver.Remote(
        #         command_executor=SafariConfig.command_executor,
        #         desired_capabilities=DesiredCapabilities.SAFARI.copy(),
        #     )

        driver = webdriver.Safari(executable_path="/usr/bin/safaridriver")
        driver.set_page_load_timeout(self.timeout)
        return driver


class SeleniumDriver(object):

    def __init__(self, browser: SeleniumBrowser):
        self.d = browser.start()

    @property
    def page_content(self):
        logger.info('get page content')
        page_source = self.d.page_source
        return page_source

    @property
    def title(self):
        logger.info("get title")
        title = self.d.title
        return title

    @property
    def url(self):
        logger.info("get url")
        url = self.d.current_url
        return url

    @property
    def alert_text(self):
        logger.info("get alert text")
        try:
            alert_text = self.d.switch_to.alert.text
        except NoAlertPresentException:
            logger.info(f'没有出现alert')
            return None
        return alert_text

    def set_page_timeout(self, timeout):
        """设置页面超时时间"""
        self.d.set_page_load_timeout(timeout)

    def set_script_time(self, timeout):
        """设置异步脚本的超时时间"""
        self.d.set_script_timeout(timeout)

    def open_url(self, url=None):
        """访问url"""
        logger.info(f"visit {url}")
        self.d.get(url)

    def add_cookies_and_refresh(self, headers: dict):
        """添加cookie后刷新页面"""
        if headers:
            cookies = [{"name": k, "value": v} for k, v in headers.items()]
            self.add_cookies(cookies)
            self.refresh()

    def back(self):
        logger.info("back")
        self.d.back()

    def screenshot(self, file_name=None, with_time=True, delay=2):
        screenshot_util(self.d, file_name=file_name, with_time=with_time,
                        delay=delay)

    def get_windows(self):
        logger.info(f"get tab list")
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info("switch to newest tab")
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    def switch_to_frame(self, frame_id):
        logger.info(f"switch to frame {frame_id}")
        self.d.switch_to.frame(frame_id)

    def frame_out(self):
        logger.info("switch frame out")
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f"exec js script: \n{script}")
        self.d.execute_script(script, *args)

    def click_elem(self, element):
        logger.info(f"click: {element}")
        self.d.execute_script("arguments[0].click();", element)

    def quit(self):
        logger.info("quit browser")
        self.d.quit()

    def close(self):
        logger.info("close current tab")
        self.d.close()

    def add_cookies(self, cookies: list):
        """添加cookie列表"""
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        """获取所有cookie"""
        logger.info("get all cookie")
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def refresh(self):
        logger.info(f"refresh page")
        self.d.refresh()

    def accept_alert(self):
        logger.info("alert accept")
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info("alert dismiss")
        self.d.switch_to.alert.dismiss()


if __name__ == '__main__':
    pass





import kuto

from page.adr_page import HomePage, \
    MyPage, SettingPage


class TestAdrDemo(kuto.Case):
    """安卓应用demo"""

    def start(self):
        self.home_page = HomePage(self.driver)
        self.my_page = MyPage(self.driver)
        self.set_page = SettingPage(self.driver)

    def test_1(self):
        self.home_page.adBtn.click_exists(timeout=10)
        self.home_page.myTab.click()
        self.my_page.settingBtn.click()
        self.assert_act('.me.MeSettingActivity')
        self.screenshot("设置页")

    def test_2(self):
        self.home_page.adBtn.click_exists(timeout=10)
        self.home_page.spaceTab.click()
        self.screenshot("科创空间首页")


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform='android',
        device_id="UJK0220521066836",
        pkg_name='com.qizhidao.clientapp',
    )





from Crypto.Cipher import AES
import base64


class KDES(object):
    """加解密方法合集"""

    def __init__(self, key, cont, _type="aes"):
        self._type = _type
        self.key = key
        self.src = cont

    @staticmethod
    def aes_encrypt(key, content):
        """
        aes加密
        @param key: 固定的16位秘钥
        @param content: 明文信息
        @return: 密文信息
        """
        aes = AES.new(str.encode(key), AES.MODE_ECB)
        encode_pwd = str.encode(content.rjust(16, '@'))
        encrypt_str = str(base64.encodebytes(aes.encrypt(encode_pwd)), encoding='utf-8')
        return encrypt_str

    @staticmethod
    def aes_decrypt(key, content):
        """
        aes解密
        @param key: 加密时传入的16位秘钥
        @param content: 密文信息
        @return: 明文信息
        """
        aes = AES.new(str.encode(key), AES.MODE_ECB)
        decrypt_str = (aes.decrypt(base64.decodebytes(content.encode(encoding='utf-8'))).decode().replace('@', ''))
        return decrypt_str

    def encrypt(self):
        if self._type == "aes":
            return self.aes_encrypt(self.key, self.src)

    def decrypt(self):
        if self._type == "aes":
            return self.aes_decrypt(self.key, self.src)


if __name__ == '__main__':
    KDES("xx", "yy").decrypt()
    pass






"""
@Author: kang.yang
@Date: 2023/9/15 10:03
"""
import kuto


class MacPage(kuto.Page):
    """根据图片定位，建议使用系统自带截图（shift+command+4）"""

    num_3 = kuto.MacElem(image='../static/calculator_3.png', desc="数字3")
    x = kuto.MacElem(image='../static/calculator_x.png', desc="乘以号")
    num_5 = kuto.MacElem(image='../static/calculator_5.png', desc="数字5")
    equal = kuto.MacElem(image='../static/calculator_=.png', desc="等于")


# class MacPage(kuto.Page):
#     """根据坐标定位，可以使用MacDriver的get_location方法实时获取"""
#     num_3 = kuto.CoorElem(coor=(1785, 364), desc="数字3")
#     x = kuto.CoorElem(coor=(1833, 369), desc="乘以号")
#     num_5 = kuto.CoorElem(coor=(1720, 316), desc="数字3")
#     equal = kuto.CoorElem(coor=(1869, 404), desc="等于")



"""
@Author: kang.yang
@Date: 2023/9/7 17:03
"""
import os

from kuto.utils.exceptions import KError
from kuto.utils.log import logger


def get_connected_adr_devices():
    """获取当前连接的手机列表"""
    cmd = 'adb devices'
    output = os.popen(cmd).read()
    device_list = [item.split('\t')[0] for item in output.split('\n') if item.endswith('device')]
    if len(device_list) > 0:
        logger.info(f"已连接设备列表: {device_list}")
        return device_list
    else:
        raise KError(msg=f"无已连接设备")



import json
import time
from typing import Union
from urllib import parse

from filelock import FileLock

from kuto.core.android.driver import AndroidDriver
from kuto.core.api.request import HttpReq
from kuto.core.ios.driver import IosDriver
from kuto.core.web.driver import PlayWrightDriver
from kuto.core.web.element import WebElem
from kuto.core.mac.driver import MacDriver
from kuto.core.win.driver import WinDriver

from kuto.utils.config import config, free_config
from kuto.utils.log import logger
from kuto.utils.exceptions import KError


class Page(object):
    """页面基类，用于pom模式封装"""

    def __init__(self, driver):
        self.driver = driver


class Case(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    driver: Union[AndroidDriver, IosDriver,
                  PlayWrightDriver, MacDriver, WinDriver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        platform = config.get_common("platform")
        if platform in ["android", "ios"]:
            # 加一段逻辑支持多进程和设备调度
            logger.info("开始获取空闲设备")
            timeout = 30  # 后面再考虑要不要在runner中加一个超时设置
            while timeout > 0:
                with FileLock("session.lock"):
                    device_id = free_config.get_random_device()
                if device_id:
                    logger.info(f"获取空闲设备成功: {device_id}")
                    logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")
                    break
                logger.info("未找到空闲设备，休息3秒")
                timeout -= 3
                time.sleep(3)
            else:
                logger.info(f"获取空闲设备失败!!!")
                logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")
                raise KError("获取空闲设备失败!!!")
            if platform == "android":
                pkg_name = config.get_app("pkg_name")
                self.driver = AndroidDriver(device_id=device_id, pkg_name=pkg_name)
            elif platform == "ios":
                pkg_name = config.get_app("pkg_name")
                self.driver = IosDriver(device_id=device_id, pkg_name=pkg_name)
        elif platform == "web":
            browserName = config.get_web("browser_name")
            headless = config.get_web("headless")
            state = config.get_web("state")
            if state:
                state_json = json.loads(state)
                self.driver = PlayWrightDriver(browserName=browserName, headless=headless, state=state_json)
            else:
                self.driver = PlayWrightDriver(browserName=browserName, headless=headless)
        elif platform == "mac":
            pkg_name = config.get_app("pkg_name")
            self.driver = MacDriver(pkg_name=pkg_name)
        elif platform == "win":
            pkg_name = config.get_app("pkg_name")
            self.driver = WinDriver(pkg_name=pkg_name)
        if isinstance(self.driver, (AndroidDriver, IosDriver, MacDriver, WinDriver)):
            if config.get_app("auto_start") is True:
                self.driver.start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if isinstance(self.driver, PlayWrightDriver):
            self.driver.close()
        if isinstance(self.driver, (AndroidDriver, IosDriver, MacDriver, WinDriver)):
            if config.get_app("auto_start") is True:
                self.driver.stop_app()
        if isinstance(self.driver, (AndroidDriver, IosDriver)):
            # 加一段逻辑支持多进程和设备调度
            device_id = self.driver.device_id
            logger.info(f"用例结束释放设备: {device_id}")
            with FileLock("session.lock"):
                devices = free_config.get('devices')
                if device_id not in devices:
                    free_config.add_devices([self.driver.device_id])
            logger.info(f"剩余空闲设备列表: {free_config.get_all_device()}")
        take_time = time.time() - self.start_time
        logger.info("用例耗时: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.info(f"暂停: {n}s")
        time.sleep(n)

    def open_url(self, url=None):
        """浏览器打开页面"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise KError('base_url is null')
            url = base_url
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise KError('base_url is null')
                url = parse.urljoin(base_url, url)
        # 访问页面
        self.driver.open_url(url)
        # 设置cookies
        cookies = config.get_web("cookies")
        if cookies:
            self.driver.set_cookies(cookies)

    def switch_tab(self, **kwargs):
        """切换到新页签，需要先定位导致跳转的元素"""
        locator = WebElem(self.driver, **kwargs)
        self.driver.switch_tab(locator)

    def screenshot(self, name: str):
        """截图"""
        self.driver.screenshot(name)

    # 断言
    def assert_act(self, activity_name: str, timeout=5):
        """断言当前activity，安卓端使用"""
        self.driver.assert_act(activity_name, timeout=timeout)

    def assert_title(self, title: str, timeout=5):
        """断言页面title，web端使用"""
        self.driver.assert_title(title, timeout=timeout)

    def assert_url(self, url: str = None, timeout=5):
        """断言页面url，web端使用"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise KError('base_url is null')
            url = base_url + "/"
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise KError('base_url is null')
                url = parse.urljoin(base_url, url)
        self.driver.assert_url(url, timeout=timeout)






import time

from kuto.core.ios.driver import IosDriver
from kuto.utils.log import logger
from kuto.core.ocr.driver import OCRDiscern
from kuto.utils.common import draw_red_by_rect

# 先上下左右分成四块，然后再上下对半分
POSITION = {
    1: "top_left",
    11: "top_left_1",
    12: "top_left_2",
    2: "top_right",
    21: "top_right_1",
    22: "top_right_2",
    3: "bottom_left",
    31: "bottom_left_1",
    32: "bottom_left_2",
    4: "bottom_right",
    41: "bottom_right_1",
    42: "bottom_right_2"
}


class OCRElem(object):
    """ocr识别定位"""

    def __init__(self,
                 driver=None,
                 text: str = None,
                 pos: int = None,
                 grade=0.8,
                 debug: bool = False):
        """
        @param driver:
        @param text:
        @param pos: 把图片分成八块，具体定义见上POSITION
        @param grade: 置信度，最大1，越高代表准确率越高
        @param debug: 是否调试
        """
        self.driver = driver
        self.text = text
        self._position = pos
        if pos is not None:
            self._position = POSITION[pos]
        self._grade = grade
        self._debug = debug

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def find_element(self, retry=3, timeout=1):
        logger.info(f"开始查找元素: {self.text}")
        for i in range(retry):
            if i > 0:
                logger.info(f"第{i}次重试")
            info = self.driver.screenshot(self.text,
                                          position=self._position)
            if self._position is not None:
                image_path = info.get("path")
            else:
                image_path = info
            res = OCRDiscern(image_path, grade=self._grade).get_coordinate(self.text)
            if isinstance(res, tuple):
                x, y = res
                if self._position is not None:
                    logger.debug(self._position)
                    width = info.get("width")
                    height = info.get("height")
                    cut_height = info.get("cut_height")

                    if 'top_right' in self._position:
                        x = width / 2 + x
                        if self._position == "top_right_2":
                            y = y + cut_height
                    elif 'top_left' in self._position:
                        if self._position == 'top_left_2':
                            y = y + cut_height
                    elif 'bottom_left' in self._position:
                        y = height / 2 + y
                        if self._position == "bottom_left_2":
                            y = y + cut_height
                    elif 'bottom_right' in self._position:
                        x = width / 2 + x
                        y = height / 2 + y
                        if self._position == "bottom_right_2":
                            y = y + cut_height
                if isinstance(self.driver, IosDriver):
                    x, y = int(x/self.driver.d.scale), int(y/self.driver.d.scale)
                logger.info(f'识别坐标为: ({x}, {y})')
                if self._debug is True:
                    file_path = self.driver.screenshot(f'ocr识别定位成功-{self.text}')
                    draw_red_by_rect(file_path, (int(x) - 100, int(y) - 100, 200, 200))
                return x, y
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'ocr识别定位失败-{self.text}')
            raise Exception('通过OCR未识别指定文字或置信度过低，无法进行点击操作！')

    def exists(self, retry=3, timeout=1):
        logger.info(f'ocr识别文本: {self.text} 是否存在')
        try:
            self.find_element(retry=retry, timeout=timeout)
        except:
            return False
        else:
            return True
        finally:
            logger.info("识别成功")

    def click(self, retry=3, timeout=1):
        logger.info(f'ocr点击文本: {self.text}')
        x, y = self.find_element(retry=retry, timeout=timeout)
        self.driver.click(x, y)
        logger.info("点击完成")


if __name__ == '__main__':
    from kuto.core.android.driver import AndroidDriver

    driver1 = AndroidDriver(device_id='UJK0220521066836', pkg_name='com.qizhidao.clientapp')
    # driver.start_app()
    elem = OCRElem(driver1, '我的')
    # all - 850, 938
    # top_right - 307, 938
    # all -
    elem.click()





"""
@Author: kang.yang
@Date: 2023/8/1 18:21
"""
import platform
import re
import cv2
import time
import allure
import functools
import os
import random
import socket

from kuto.utils.log import logger


def draw_red_by_rect(image_path: str, rect: tuple):
    """x_top_left, y_top_left, width, height"""
    # 读取图像
    image = cv2.imread(image_path)

    # 定义标记范围的坐标
    x, y, w, h = rect

    # 在图像上绘制矩形
    cv2.rectangle(image, (x, y), (x + w, y + h), (0, 0, 255), 2)

    # 保存标记后的图像
    cv2.imwrite(image_path, image)


def draw_red_by_coordinate(image_path: str, rect: tuple):
    """x_top_left, y_top_left, x_bottom_right, y_bottom_right"""
    # 读取图像
    image = cv2.imread(image_path)

    # 定义标记范围的坐标
    x, y, x1, y1 = rect

    # 在图像上绘制矩形
    cv2.rectangle(image, (x, y), (x1, y1), (0, 0, 255), 2)

    # 保存标记后的图像
    cv2.imwrite(image_path, image)


def is_port_in_use(port: int) -> bool:
    """判断端口是否已占用"""
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('127.0.0.1', port)) == 0


def get_free_port():
    """获取空闲端口"""
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('127.0.0.1', 0))
        try:
            return s.getsockname()[1]
        finally:
            s.close()
    except OSError:
        # bind 0 will fail on Manjaro, fallback to random port
        # https://github.com/openatx/adbutils/issues/85
        for _ in range(20):
            port = random.randint(10000, 20000)
            if not is_port_in_use(port):
                return port
        raise RuntimeError("No free port found")


def cut_half(image, position):
    """把图片分成上下两半"""
    if position == "up":
        return image[:image.shape[0] // 2, :]
    elif position == "down":
        return image[image.shape[0] // 2:, :]
    else:
        raise KeyError("position传值错误")


def cut_by_position(image_path: str, position: str):
    """把图片按左上、左下、右上、右下进行分割"""
    logger.info(position)
    # 读取图像
    logger.info("分割图片")
    start = time.time()
    image = cv2.imread(image_path)
    # 获取图像的宽度和高度
    height, width, _ = image.shape
    logger.debug(f'{height}, {width}')
    # 计算每个切割区域的宽度和高度
    sub_width = width // 2
    sub_height = height // 2
    # 切割图像成上下左右四个等份
    if "top_left" in position:
        image_content = image[0:sub_height, 0:sub_width]
        if position == "top_left_1":
            image_content = cut_half(image_content, "up")
        elif position == "top_left_2":
            image_content = cut_half(image_content, "down")
    elif "top_right" in position:
        image_content = image[0:sub_height, sub_width:width]
        if position == "top_right_1":
            image_content = cut_half(image_content, "up")
        elif position == "top_right_2":
            image_content = cut_half(image_content, "down")
    elif "bottom_left" in position:
        image_content = image[sub_height:height, 0:sub_width]
        if position == "bottom_left_1":
            image_content = cut_half(image_content, "up")
        elif position == "bottom_left_2":
            image_content = cut_half(image_content, "down")
    elif "bottom_right" in position:
        image_content = image[sub_height:height, sub_width:width]
        if position == "bottom_right_1":
            image_content = cut_half(image_content, "up")
        elif position == "bottom_right_2":
            image_content = cut_half(image_content, "down")
    else:
        raise KeyError(f"position传值错误 all: {position}")

    new_path = f"{image_path.split('.')[0]}_{position}.{image_path.split('.')[1]}"
    logger.debug(new_path)
    cv2.imwrite(new_path, image_content)
    cut_height, cut_width, _ = image_content.shape
    logger.debug(f'{cut_height, cut_width}')
    end = time.time()
    logger.info(f"分割成功， 耗时: {end - start}s")
    info = {
        "path": new_path,
        "height": height,
        "width": width,
        "cut_height": cut_height
    }
    logger.debug(info)
    return info


def screenshot_util(driver, file_name=None, position: str = None):
    """
    截图并保存到预定路径
    @param driver
    @param file_name: foo.png or fool
    @param position: top_left\top_right\bottom_left\bottom_right
    @return:
    """
    logger.info("开始截图")
    if not file_name:
        raise ValueError("文件名不能为空")

    # 截图并保存到当前目录的image文件夹中
    relative_path = "screenshot"
    # 把文件名处理成test.png的样式
    if "." in file_name:
        file_name = file_name.split(r".")[0]
    if os.path.exists(relative_path) is False:
        os.mkdir(relative_path)
    time_str = time.strftime(f"%Y%m%d%H%M%S")
    file_name = f"{time_str}_{file_name}.jpg"

    file_path = os.path.join(relative_path, file_name)
    try:
        driver.screenshot(file_path)
    except:
        driver.screenshot(path=file_path)
    logger.info(f"保存至: {file_path}")
    # 对图片进行分割
    info = None
    if position is not None:
        info = cut_by_position(file_path, position)
        file_path = info.get("path")
    # 上传allure报告
    allure.attach.file(
        file_path,
        attachment_type=allure.attachment_type.PNG,
        name=f"{file_path}",
    )
    if position is not None:
        return info
    else:
        return file_path


def sanitize_filename(filename):
    """去除文件名中的非法字符"""
    # Windows 非法字符：\/:*?"<>|
    windows_illegal_chars = r'[\\/:\*\?"<>|]'
    # macOS 非法字符：/
    macos_illegal_chars = r'/'
    # Linux 非法字符：/
    linux_illegal_chars = r'/'

    # 根据当前操作系统选择对应的非法字符规则
    if platform.system() == 'Windows':
        illegal_chars = windows_illegal_chars
    elif platform.system() == 'Darwin':
        illegal_chars = macos_illegal_chars
    else:
        illegal_chars = linux_illegal_chars

    # 移除非法字符
    filename = re.sub(illegal_chars, '', filename)
    # 替换空格为下划线
    filename = filename.replace(' ', '_')
    # 去掉末尾的点号和空格
    filename = filename.rstrip('. ')
    return filename


def dict_to_str(source_dict: dict):
    """把字典转化成字符串，并调整成合规的文件名"""
    dict_str = '_'.join([sanitize_filename(f"{key}") for key, value in source_dict.items()])
    return dict_str


def calculate_time(func):
    """计算方法耗时"""

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        execution_time = end_time - start_time
        logger.info(f"函数 {func.__name__} 的执行时间为: {execution_time} 秒")
        return result
    return wrapper


if __name__ == '__main__':
    print(get_free_port())






"""
@Author: kang.yang
@Date: 2023/9/20 10:53
"""
import kuto


class WinPage(kuto.Page):
    num_3 = kuto.MacElem(image='../static/calculator_3_win.png', desc="数字3")
    x = kuto.MacElem(image='../static/calculator_x_win.png', desc="乘以号")
    num_5 = kuto.MacElem(image='../static/calculator_5_win.png', desc="数字5")
    equal = kuto.MacElem(image='../static/calculator_=_win.png', desc="等于")



import kuto

from page.ios_page import *


class TestIosDemo(kuto.Case):
    """IOS应用demo"""

    def start(self):
        self.index_page = IndexPage(self.driver)
        self.my_page = MyPage(self.driver)
        self.set_page = SettingPage(self.driver)

    def test_go_setting(self):
        self.index_page.myTab.click()
        self.my_page.settingBtn.click()
        self.set_page.about.assert_exists()


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(
        platform="ios",
        device_id="00008101-000E646A3C29003A",
        pkg_name='com.qizhidao.company'
    )





import time
import hmac
import hashlib
import base64
import urllib.parse
import requests
import json

from kuto.utils.allure_util import get_allure_data
from kuto.utils.log import logger


# 钉钉机器人
class DingTalk:
    """消息格式参考：https://open.dingtalk.com/document/robots/custom-robot-access"""

    def __init__(self, secret=None, url=None):
        self.secret = secret
        self.url = url

    # 生成时间戳timestamp和签名数据sign用于钉钉机器人的请求
    def __gen_timestamp_and_sign(self):
        timestamp = str(round(time.time() * 1000))
        secret_enc = self.secret.encode('utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, self.secret)
        string_to_sign_enc = string_to_sign.encode('utf-8')
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return timestamp, sign

    # 发送钉钉机器人通知
    def send_msg(self, msg_data):
        """
        参考：https://open.dingtalk.com/document/robots/custom-robot-access
        @param msg_data:
        @return:
        """
        # 从gen_timestamp_and_sign方法获取timestamp和sign
        timestamp, sign = self.__gen_timestamp_and_sign()
        # 机器人url
        robot_url = self.url
        # 拼接请求url
        url = '{0}&timestamp={1}&sign={2}'.format(robot_url, timestamp, sign)
        print(url)
        # 请求头
        headers = {'Content-Type': 'application/json'}
        # 发送请求
        ret = requests.post(url, headers=headers, data=json.dumps(msg_data), verify=False)
        # 判断请求结果
        ret_dict = ret.json()
        if ret_dict.get('errcode') == 0:
            print('消息发送成功')
        else:
            print('消息发送失败: {}'.format(ret_dict.get('errmsg')))

    # 从allure报告中获取数据并发送消息
    def send_report(self, msg_title, report_url):
        allure_data = get_allure_data('report')
        total = allure_data.get('total')
        passed = allure_data.get('passed')
        fail = allure_data.get('failed')
        rate = allure_data.get('rate')

        color_red = 'FF0000'
        color_green = '00FF00'
        if rate < 100:
            color_str = color_red
        else:
            color_str = color_green

        msg_title = f'{msg_title}({time.strftime("%m-%d %H:%M")})'
        result_text = "### {0}\n\n" \
                      "<font color=#C0C0C0>总数：</font>{1}，" \
                      "<font color=#C0C0C0>通过：</font><font color=#C0C0C0>{2}，</font>\n" \
                      "<font color=#C0C0C0>失败：</font><font color=#C0C0C0>{3}，</font>\n" \
                      "<font color=#C0C0C0>成功率：</font><font color=#{4}>{5}%</font>\n" \
                      "> #### [查看详情]({6})\n".format(msg_title, total, passed, fail, color_str, rate, report_url)
        msg_data = {
            "msgtype": "markdown",
            "markdown": {
                "title": msg_title,
                "text": result_text
            }
        }

        if fail > 0:
            self.send_msg(msg_data)
        else:
            logger.info('用例全部成功，无需通知~')







import os
import yaml
import random
import time


from kuto.utils.log import logger


local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.yml')

    def get(self, module, key):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        return yaml_data[module][key]

    def get_api(self, key):
        return self.get('api', key)

    def get_app(self, key):
        return self.get('app', key)

    def get_web(self, key):
        return self.get('web', key)

    def get_common(self, key):
        return self.get('common', key)

    def set(self, module, key, value):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        yaml_data[module][key] = value
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(yaml_data, f)

    def set_api(self, key, value):
        self.set('api', key, value)

    def set_app(self, key, value):
        self.set('app', key, value)

    def set_web(self, key, value):
        self.set('web', key, value)

    def set_common(self, key, value):
        self.set('common', key, value)


class FreeConfig:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'free_devices.yml')

    def get(self, key):
        try:
            with open(self.file_path, "r", encoding="utf-8") as f:
                yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
            return yaml_data[key]
        except:
            return []

    def set(self, key, value):
        with open(self.file_path, "r", encoding="utf-8") as f:
            yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
        yaml_data[key] = value
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(yaml_data, f)

    def add_devices(self, devices: list):
        old_devices = self.get("devices")
        new_devices = old_devices + devices
        self.set("devices", new_devices)

    def set_devices(self, devices: list):
        self.set("devices", devices)

    def get_all_device(self):
        return self.get("devices")

    def get_random_device(self):
        devices: list = self.get_all_device()
        if devices:
            device = random.choice(devices)
            devices.remove(device)
            self.set_devices(devices)
            return device
        return []

    def clear_devices(self):
        new_devices = []
        self.set("devices", new_devices)


config = Config()
free_config = FreeConfig()


if __name__ == '__main__':
    print(free_config.get_random_device(timeout=5))
    free_config.add_devices(["4"])
    print(free_config.get("devices"))
    print(free_config.get_random_device(timeout=5))
    print(free_config.clear_devices())












import subprocess
import sys
import time
import requests

from selenium import webdriver
from selenium.common.exceptions import NoAlertPresentException

from kuto.core.android.driver import AndroidDriver
from kuto.utils.common import screenshot_util
from kuto.utils.log import logger
from kuto.utils.webdriver_manager_extend import ChromeDriverManager
from kuto.testdata import get_int


class ChromeConfig:
    headless = False
    options = None
    command_executor = ""


def get_server_chrome_versions():
    """获取淘宝镜像网站上所有的版本列表"""
    version_list = []
    url = "https://registry.npmmirror.com/-/binary/chromedriver/"
    rep = requests.get(url).json()
    for item in rep:
        version_list.append(item["name"])
    return version_list


def get_webview_version(serial_no, pkg_name):
    """通过shell命令获取webview对应chrome的版本号"""
    # 获取应用的pid
    pid = subprocess.getoutput(f"adb -s {serial_no} shell ps | grep {pkg_name}").split(
        " "
    )[6]
    print(pid)

    # 获取webview的socket名称
    socket_name = subprocess.getoutput(
        f"adb -s {serial_no} shell cat /proc/net/unix | grep --binary-file=text webview"
    ).split(" ")[-1][1:]
    print(socket_name)

    # 将webview端口转发到本地
    cmd = f"adb -s {serial_no} forward tcp:5000 localabstract:{socket_name}"
    print(cmd)
    subprocess.getoutput(cmd)

    # 请求本地服务，获取版本号
    version_data = requests.get("http://127.0.0.1:5000/json/version").json()
    print(version_data)
    version = version_data["Browser"].split("/")[1]
    print(version)

    return version


def get_driver_version(serial_no, pkg_name):
    """获取应用对应的淘宝镜像的chromedriver的版本号"""
    webview_version = get_webview_version(serial_no, pkg_name)

    version_list = get_server_chrome_versions()
    if webview_version in version_list:
        return webview_version
    else:
        for version in version_list:
            if version[0:10] == webview_version[0:10]:
                return version.split("/")[0]
        else:
            print("can not found right version in taobao")
            sys.exit()


class H5Driver(object):

    def __init__(self, android_driver: AndroidDriver, timeout=60):
        serial_no = android_driver.device_id
        pkg_name = android_driver.pkg_name

        logger.info(f"start H5Driver for {serial_no}")
        options = webdriver.ChromeOptions()
        options.add_experimental_option("androidDeviceSerial", serial_no)
        options.add_experimental_option("androidPackage", pkg_name)
        options.add_experimental_option("androidUseRunningApp", True)
        options.add_experimental_option("androidProcess", pkg_name)
        for i in range(3):
            logger.debug(f"{i + 1} time try find:")
            try:
                version = get_driver_version(serial_no, pkg_name)
                self.d = webdriver.Chrome(
                    executable_path=ChromeDriverManager(version=version).install(),
                    options=options,
                )
                logger.debug("h5Driver init success.")
                break
            except Exception as e:
                logger.debug(f"{i + 1} times retry")
                time.sleep(3)
        else:
            logger.debug("retry 3 times，h5Driver init fail.")
            sys.exit()

        # 设置页面加载超时时间
        self.d.set_page_load_timeout(timeout)

    @property
    def page_content(self):
        page_source = self.d.page_source
        logger.info(f"get page content: \n{page_source}")
        return page_source

    @property
    def title(self):
        logger.info("get page title")
        title = self.d.title
        logger.info(title)
        return title

    @property
    def url(self):
        logger.info("get page url")
        url = self.d.current_url
        logger.info(url)
        return url

    @property
    def alert_text(self):
        logger.info("get alert text")
        try:
            alert_text = self.d.switch_to.alert.text
        except NoAlertPresentException:
            logger.info(f'no alert')
            return None
        return alert_text

    def open_url(self, url):
        logger.info(f"visit: {url}")
        self.d.get(url)

    def back(self):
        logger.info("back")
        self.d.back()

    def screenshot(self, file_name=None):
        screenshot_util(self.d, file_name=file_name)

    def get_windows(self):
        logger.info(f"get current tab")
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info("switch to newest tab")
        current_windows = self.d.window_handles
        newest_window = [
            window for window in current_windows if window not in old_windows
        ][0]
        self.d.switch_to.window(newest_window)

    def switch_to_frame(self, frame_id):
        logger.info(f"switch to frame {frame_id}")
        self.d.switch_to.frame(frame_id)

    def switch_to_frame_out(self):
        logger.info("frame out")
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f"exec js script: \n{script}")
        self.d.execute_script(script, *args)

    def click_elem(self, element):
        logger.info(f"click: {element}")
        self.d.execute_script("arguments[0].click();", element)

    def accept_alert(self):
        logger.info("alert accept")
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info("alert dismiss")
        self.d.switch_to.alert.dismiss()



import kuto
from kuto import logger


LIST_DATA = [
    {"name": "李雷", "age": "33"},
    {"name": "韩梅梅", "age": "30"}
]


class TestParameter(kuto.Case):
    """
    参数化demo
    """

    @kuto.data(LIST_DATA)
    def test_list(self, param):
        logger.info(param)

    @kuto.file_data(file='../static/data.json')
    def test_json(self, param):
        logger.info(param)

    @kuto.file_data(file='../static/data.yml', key='names')
    def test_yaml(self, param):
        print(param)

    @kuto.file_data(file='../static/data.csv')
    def test_csv(self, param):
        logger.info(param)

    @kuto.file_data(file='../static/data.xlsx', row=1)
    def test_excel(self, param):
        logger.info(param)


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main()



import logging
from loguru import logger


class PropogateHandler(logging.Handler):
    def emit(self, record) -> None:
        logging.getLogger(record.name).handle(record)


logger.add(PropogateHandler(), format="{time:YYYY-MM-DD at HH:mm:ss} | {message}")


# # 创建一个logger
# logger = logging.getLogger()
# logger.setLevel(logging.DEBUG)
#
# # 再创建一个handler，用于输出到控制台
# ch = logging.StreamHandler(sys.stdout)
# ch.setLevel(logging.DEBUG)
#
# # 定义handler的输出格式
# formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
# ch.setFormatter(formatter)
#
# # 给logger添加handler
# if not logger.handlers:
#     logger.addHandler(ch)




"""
@Author: kang.yang
@Date: 2023/8/21 17:05
"""
import kuto

"""
ocr识别可以配合安卓应用或者IOS应用进行使用
"""


class OcrPage(kuto.Page):
    searchBtn = kuto.IosElem(text="搜索", className="XCUIElementTypeSearchField", desc='搜索框入口')
    searchInput = kuto.IosElem(className="XCUIElementTypeSearchField", desc='搜索框')
    searchResult = kuto.IosElem(xpath="//Table/Cell[2]", desc='搜索结果')
    schoolEntry = kuto.OCRElem(text="校园场馆", pos=12)




import os
import cv2
import numpy as np


class ImageDiscern:
    def __init__(self, target_image, source_image, grade=0.8, gauss_num=111) -> None:
        """__init__ [处理图片识别]
        Args:
            target_image (str): 被识别的目标图片
            grade (float, optional): 分数. Defaults to 0.8.
            gauss_num (int, optional): 过滤值. Defaults to 111.
        """
        self.source_image = source_image
        self.target_image = target_image
        if not os.path.exists(self.source_image):
            raise FileNotFoundError(f"文件: {self.source_image} 不存在")
        if not os.path.exists(self.target_image):
            raise FileNotFoundError(f"文件: {self.target_image} 不存在")

        self.grade = grade
        self.gauss_num = gauss_num

    # 降噪处理（高斯滤波）
    def __coordinate(self, image):
        return cv2.GaussianBlur(image, (self.gauss_num, self.gauss_num), 0)

    # 获取坐标图片坐标
    def get_coordinate(self):
        # 这里不用imread，解决中文目录无法读取的问题
        screen = cv2.imdecode(np.fromfile(self.source_image, dtype=np.uint8), 1)
        target = cv2.imdecode(np.fromfile(self.target_image, dtype=np.uint8), 1)
        result = cv2.matchTemplate(self.__coordinate(screen), self.__coordinate(target), cv2.TM_CCOEFF_NORMED)
        min, max, min_loc, max_loc = cv2.minMaxLoc(result)
        if max <= self.grade:
            return False
        else:
            x = max_loc[0] + int(target.shape[1] / 2)
            y = max_loc[1] + int(target.shape[0] / 2)
            return (x, y)






"""
@Author: kang.yang
@Date: 2023/5/13 10:16
"""
from playwright.sync_api import expect

from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.core.web.driver import PlayWrightDriver
from kuto.utils.common import calculate_time


class WebElem:
    """
    通过selenium定位的web元素
    """

    def __init__(self,
                 driver: PlayWrightDriver = None,
                 desc: str = None,
                 xpath: str = None,
                 css: str = None,
                 text: str = None,
                 holder: str = None,
                 index: int = None,
                 debug: bool = False):
        """

        @param driver: 浏览器驱动]
        @param desc: 元素描述
        @param xpath: xpath
        @param css: css selector
        @param text: 页面内容中的所有文本
        @param holder: 输入框默认文案
        @param debug: 是否调试
        """
        self._driver = driver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = holder
        self._index = index
        self._debug = debug

        self._kwargs = {}
        if self._xpath is not None:
            self._kwargs["xpath"] = self._xpath
        if self._css is not None:
            self._kwargs["css"] = self._css
        if self._text is not None:
            self._kwargs["text"] = text
        if self._placeholder is not None:
            self._kwargs["placeholder"] = self._placeholder
        if self._index is not None:
            self._kwargs["index"] = self._index

    def __get__(self, instance, owner):
        """pm模式的关键"""
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    # 公共方法
    @calculate_time
    def find(self, timeout=10):
        """查找指定的一个元素"""
        element = None
        if self._text:
            logger.info(f"根据文本定位: {self._text}")
            element = self._driver.page.get_by_text(self._text)
        if self._placeholder:
            logger.info(f"根据输入框默认文本定位: {self._placeholder}")
            element = self._driver.page.get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"根据css selector定位: {self._css}")
            element = self._driver.page.locator(self._css)
        if self._xpath:
            logger.info(f"根据xpath定位: {self._xpath}")
            element = self._driver.page.locator(self._xpath)
        if self._index:
            element = element.nth(self._index)
        try:
            element.wait_for(timeout=timeout*1000)
            logger.info("查找成功")
            if self._debug is True:
                element.evaluate('(element) => element.style.border = "2px solid red"')
                time.sleep(1)
                self._driver.screenshot(self._desc + "_查找成功")
            return element
        except:
            logger.info("查找失败")
            self._driver.screenshot(self._desc + "_查找失败")
            raise KError(f"{self._kwargs} 查找失败")

    # 属性
    @property
    def text(self):
        logger.info(f"获取 {self._desc} 文本属性")
        elems = self.find().all()
        text = [elem.text_content() for elem in elems]
        logger.info(text)
        return text

    # 其他方法
    def click(self, timeout=5):
        logger.info(f"点击 {self._desc}")
        self.find(timeout=timeout).click()

    def input(self, text, timeout=5):
        logger.info(f"输入文本: {text}")
        self.find(timeout=timeout).fill(text)

    def check(self, timeout=5):
        logger.info("选择选项")
        self.find(timeout=timeout).check()

    def select(self, value: str, timeout=5):
        logger.info("下拉选择")
        self.find(timeout=timeout).select_option(value)

    def assert_visible(self, timeout=5):
        logger.info(f"断言 {self._desc} 可见")
        expect(self.find(timeout=timeout)).to_be_visible()

    def assert_hidden(self, timeout=5):
        logger.info(f"断言 {self._desc} 被隐藏")
        expect(self.find(timeout=timeout)).to_be_hidden()

    def assert_text_cont(self, text: str, timeout=5):
        logger.info(f"断言 {self._desc} 包含文本: {text}")
        expect(self.find(timeout=timeout)).to_contain_text(text)

    def assert_text_eq(self, text: str, timeout=5):
        logger.info(f"断言 {self._desc} 文本等于: {text}")
        expect(self.find(timeout=timeout)).to_have_text(text)


class FraElem(WebElem):

    def __init__(self, driver: PlayWrightDriver = None, frame: str = None, desc: str = None, xpath: str = None, css: str = None,
                 text: str = None, holder: str = None, index: int = None):
        """

        @param driver: 浏览器驱动
        @type frame: frame定位方式，使用正常的css定位方式即可
        @param xpath: 根据xpath进行定位
        @param css: 根据css selector进行定位
        @param text: 根据标签的文本定位
        @param holder: 根据输入框的placeholder定位
        """
        super().__init__(driver, desc, xpath, css, text, holder, index)
        self._driver = driver
        if not desc:
            raise KError("frame元素描述不能为空")
        else:
            self.desc = desc
        self._frame_loc = frame
        self._xpath = xpath
        self._css = css
        self._text = text
        self._placeholder = holder
        self._index = index

    @calculate_time
    def find(self, timeout=5):
        element = None
        if self._text:
            logger.info(f"根据文本定位: {self._text}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_text(self._text)
        if self._placeholder:
            logger.info(f"根据输入框默认文本定位: {self._placeholder}")
            element = self._driver.page.frame_locator(self._frame_loc).get_by_placeholder(self._placeholder)
        if self._css:
            logger.info(f"根据css定位: {self._css}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._css)
        if self._xpath:
            logger.info(f"根据xpath定位: {self._xpath}")
            element = self._driver.page.frame_locator(self._frame_loc).locator(self._xpath)
        if self._index:
            element = element.nth(self._index)

        try:
            element.wait_for(timeout=timeout*1000)
            logger.info("查找成功")
            return element
        except:
            logger.info("查找失败")
            self._driver.screenshot(self._desc + "_查找失败")
            raise KError(f"{self._kwargs} 查找失败")


if __name__ == '__main__':
    import time

    driver1 = PlayWrightDriver(browserName="chrome")
    driver1.open_url("https://www-test.qizhidao.com")
    WebElem(driver1, text='登录/注册').click()
    time.sleep(5)



"""
@Author: kang.yang
@Date: 2023/8/21 17:10
"""
import kuto

"""
图像识别可以配合安卓应用或者IOS应用使用
"""


class ImagePage(kuto.Page):
    searchEntry = kuto.AdrElem(rid="com.tencent.mm:id/j5t", desc='搜索框入口')
    searchInput = kuto.AdrElem(rid="com.tencent.mm:id/cd7", desc='搜索框')
    searchResult = kuto.AdrElem(rid="com.tencent.mm:id/kpm", desc="搜索结果")
    schoolEntry = kuto.ImageElem(image="../static/校园场馆.png", desc='校园场馆入口')





import kuto


class TestApiDemo(kuto.Case):
    """接口demo"""

    @kuto.title("一般请求")
    def test_normal_req(self):
        payload = {"type": 2}
        headers = {
            "user-agent-web": "X/b67aaff2200d4fc2a2e5a079abe78cc6"
        }
        self.post('/qzd-bff-app/qzd/v1/home/getToolCardListForPc',
                  json=payload, headers=headers)
        self.assert_eq('code', 0)

    @kuto.title("文件上传")
    def test_upload_file(self):
        path = '/qzd-bff-patent/patent/batch/statistics/upload'
        files = {'static': open('../static/号码上传模板_1.xlsx', 'rb')}
        self.post(path, files=files)
        self.assert_eq('code', 0)

    @kuto.title("form请求")
    def test_form_req(self):
        url = '/qzd-bff-patent/image-search/images'
        file_data = (
            'logo.png',
            open('../static/logo.png', 'rb'),
            'image/png'
        )
        fields = {
            'key1': 'value1',  # 参数
            'imageFile': file_data  # 文件
        }
        form_data = kuto.MultipartEncoder(fields=fields)
        headers = {'Content-Type': form_data.content_type}
        self.post(url, data=form_data, headers=headers)
        self.assert_eq("code", 0)


if __name__ == '__main__':
    """仅执行本模块"""
    kuto.main(host='https://app-test.qizhidao.com')



import os
import subprocess

from kuto.utils.log import logger
from kuto.utils.exceptions import KError


def get_connected_ios_devices():
    """获取当前连接的设备列表"""
    cmd = 'tidevice list'
    output = os.popen(cmd).read()
    device_list = [item.split(' ')[0] for item in output.split('\n') if item]
    if len(device_list) > 0:
        logger.info(f"已连接设备列表: {device_list}")
        return device_list
    else:
        raise KError(msg=f"无已连接设备")


class TideviceUtil:
    """
    tidevice常用功能的封装
    """

    @staticmethod
    def uninstall_app(device_id=None, pkg_name=None):
        """卸载应用"""
        cmd = f"tidevice -u {device_id} uninstall {pkg_name}"
        logger.info(f"卸载应用: {pkg_name}")
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{device_id} 卸载应用{pkg_name} 成功")
            return
        else:
            logger.info(f"{device_id} 卸载应用{pkg_name}失败，因为{output}")

    @staticmethod
    def install_app(device_id=None, ipa_url=None):
        """安装应用
        """
        cmd = f"tidevice -u {device_id} install {ipa_url}"
        logger.info(f"安装应用: {ipa_url}")
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{device_id} 安装应用{ipa_url} 成功")
            return
        else:
            logger.info(f"{device_id} 安装应用{ipa_url}失败，因为{output}")





import os.path
import sys


case_content_android = """import kuto
from kuto import AdrElem


class HomePage(kuto.Page):
    ad = AdrElem(resourceId='com.qizhidao.clientapp:id/bottom_btn')
    my = AdrElem(text='我的')


class MyPage(kuto.Page):
    setting = AdrElem(resourceId='com.qizhidao.clientapp:id/me_top_bar_setting_iv')


class TestSetting(kuto.TestCase):

    def start(self):
        self.home_page = HomePage(self.driver)
        self.my_page = MyPage(self.driver)

    def test_1(self):
        self.home_page.ad.click_exists(timeout=5)
        self.home_page.my.click()
        self.my_page.setting.click()


if __name__ == '__main__':
    kuto.main(did='UJK0220521066836', pkg='com.qizhidao.clientapp')

"""

case_content_ios = """import kuto
from kuto import IosElem


class IndexPage(kuto.Page):
    ad = IosElem(text='close white big')
    my = IosElem(text='我的')


class MyPage(kuto.Page):
    setting = IosElem(text='settings navi')


class TestSearch(kuto.TestCase):

    def start(self):
        self.index_page = IndexPage(self.driver)
        self.my_page = MyPage(self.driver)

    def test_1(self):
        self.index_page.ad.click_exists(timeout=5)
        self.index_page.my.click()
        self.my_page.setting.click()


if __name__ == '__main__':
    kuto.main(did='00008101-000E646A3C29003A', pkg='com.qizhidao.company')

"""

case_content_web = """import kuto
from kuto import WebElem


class IndexPage(kuto.Page):
    login = WebElem(text='登录/注册')
    enterprise = WebElem(text='查专利')


class LoginPage(kuto.Page):
    pwdLogin = WebElem(text='帐号密码登录')
    userInput = WebElem(placeholder='请输入手机号码')
    pwdInput = WebElem(placeholder='请输入密码')
    licenseBtn = WebElem(css="span.el-checkbox__inner", index=1)
    loginBtn = WebElem(text='立即登录')


class TestLogin(kuto.TestCase):

    def start(self):
        self.index_page = IndexPage(self.driver)
        self.login_page = LoginPage(self.driver)

    def test_1(self):
        self.open()
        self.index_page.login.click()
        self.login_page.pwdLogin.click()
        self.login_page.userInput.input('xxx')
        self.login_page.pwdInput.input('xxx')
        self.login_page.licenseBtn.click()
        self.login_page.loginBtn.click()
        self.screenshot("登录成功")


if __name__ == '__main__':
    # with open("state.json", "r") as f:
    #     state = json.loads(f.read())

    kuto.main(browser="chrome", host="https://www.qizhidao.com")

"""

case_content_api = """import kuto


class TestHome(kuto.TestCase):

    def test_1(self):
        payload = {"type": 2}
        headers = {"user-agent-web": "X/b67aaff2200d4fc2a2e5a079abe78cc6"}
        self.post('/qzd-bff-app/qzd/v1/home/getToolCardListForPc', json=payload, headers=headers)
        self.assertEq('code', 0)


if __name__ == '__main__':
    kuto.main(
        host='https://app.qizhidao.com'
    )

"""


def create_scaffold(platform):
    """create scaffold with specified project name."""

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    # 新增测试数据目录
    root_path = f"{platform}_demo"
    create_folder(root_path)
    create_folder(os.path.join(root_path, "tests"))
    create_folder(os.path.join(root_path, "report"))
    create_folder(os.path.join(root_path, "data"))
    if platform in ["android", "ios", "web"]:
        create_folder(os.path.join(root_path, "screenshot"))
    # 新增安卓测试用例
    if platform == "android":
        # 新增安卓测试用例
        create_file(
            os.path.join(root_path, "tests", "test_adr.py"),
            case_content_android,
        )

    elif platform == "ios":
        # 新增ios测试用例
        create_file(
            os.path.join(root_path, "tests", "test_ios.py"),
            case_content_ios,
        )
    elif platform == "web":
        # 新增web测试用例
        create_file(
            os.path.join(root_path, "tests", "test_web.py"),
            case_content_web,
        )
    elif platform == "api":
        # 新增接口测试用例
        create_file(
            os.path.join(root_path, "tests", "test_api.py"),
            case_content_api,
        )
    else:
        print("请输入正确的平台: android、ios、web、api")
        sys.exit()



import pandas as pd


class Excel(object):
    """excel读写"""

    def __init__(self, file_name):
        self.file_name = file_name

    def read_all(self, sheet_name=0):
        df = pd.read_excel(self.file_name, sheet_name=sheet_name)
        res = df.values.tolist()
        return res

    def read_row_index(self, row_index: int, sheet_name=0):
        """
        index：第一行（index=0）需要有标题，默认会忽略，取值从1开始
        """
        df = pd.read_excel(self.file_name, sheet_name=sheet_name)
        res = df.values[row_index-1].tolist()
        return res

    def read_col_index(self, col_index: int, sheet_name=0):
        """
        index：从1开始
        """
        df = pd.read_excel(self.file_name, usecols=[col_index-1], sheet_name=sheet_name)
        res = [r[0] for r in df.values.tolist()]
        return res

    def read_col_name(self, col_name: str, sheet_name=0):
        df = pd.read_excel(self.file_name, sheet_name=sheet_name)
        res = df[col_name].values.tolist()
        return res

    def write(self, data: dict, sheet_name='sheet1', column_width=20):
        """
        数据格式：{
            '标题列1': ['张三', '李四'],
            '标题列2': [80, 90]
        }
        """

        df = pd.DataFrame(data)
        writer = pd.ExcelWriter(self.file_name)
        df.to_excel(writer, sheet_name=sheet_name, index=False)
        sheet = writer.sheets.get(sheet_name)
        for index in range(len(df)):
            # print(index, value)
            for i in range(len(df.columns)):
                sheet.set_column(index+1, i, column_width)
        writer.save()

    def write_sheets(self, sheet_dict: dict, column_width=20):
        """
        sheet_dict: {
            'sheet1_name': {'标题列1': ['张三', '李四'], '标题列2': [80, 90]},
            'sheet2_name': {'标题列3': ['王五', '郑六'], '标题列4': [100, 110]}
        }
        """
        writer = pd.ExcelWriter(self.file_name)
        for sheet_name, sheet_data in sheet_dict.items():
            df = pd.DataFrame(sheet_data)
            df.to_excel(writer, sheet_name=sheet_name, index=False)
            sheet = writer.sheets.get(sheet_name)
            for index in range(len(df)):
                for i in range(len(df.columns)):
                    sheet.set_column(index + 1, i, column_width)
        writer.save()

    def get_sheet_names(self):
        return list(pd.read_excel(self.file_name, sheet_name=None))


class CSV(object):
    """csc读写"""

    def __init__(self, file_name):
        self.file_name = file_name

    def read_all(self):
        df = pd.read_csv(self.file_name)
        res = df.values.tolist()
        return res

    def read_row_index(self, row_index: int):
        """
        index: 第一行（index=0）需要有标题，默认会忽略，取值从1开始
        """
        df = pd.read_csv(self.file_name)
        res = df.values[row_index-1].tolist()
        return res

    def read_col_index(self, col_index: int):
        """
        index：从1开始
        """
        df = pd.read_csv(self.file_name, usecols=[col_index-1])
        res = [r[0] for r in df.values.tolist()]
        return res

    def read_col_name(self, col_name: str):
        df = pd.read_csv(self.file_name, usecols=[col_name])
        res = [r[0] for r in df.values.tolist()]
        return res

    def write(self, data: dict):
        """
        数据格式：{
            '标题列1': ['张三', '李四'],
            '标题列2': [80, 90]
        }
        """
        df = pd.DataFrame(data)
        df.to_csv(self.file_name, index=False)


if __name__ == '__main__':
    pass




import os
import ssl
import time
import easyocr
import cv2

from kuto.utils.log import logger

os.environ["CUDA_VISIBLE_DEVICES"] = "True"
os.environ["KMP_DUPLICATE_LIB_OK"] = "True"

ssl._create_default_https_context = ssl._create_unverified_context


class OCRDiscern:
    """
        通过OCR进行文字识别，获取相应坐标值
    """

    def __init__(self, image_path: str, grade=0.8) -> None:
        # self.image_path = os.path.join(os.path.abspath('image'), 'SourceImage.png')
        self.image_path = image_path
        if not os.path.exists(self.image_path):
            raise FileNotFoundError(f"文件: {self.image_path} 不存在")
        self.model = ['ch_sim', 'en']
        self.grade = grade
        # self.coordinate_set = self.__ocr_read_wording()
        # logger.debug(self.coordinate_set)

    # 将图片缩放至450P
    def __resize_image(self):
        logger.info("缩放图片")
        # 读取图像
        image = cv2.imread(self.image_path)

        # 缩放比例
        scale_percent = 450 / image.shape[1] * 100  # 缩放比例为高为450时的缩放比例

        # 计算缩放后的图像尺寸
        width = int(image.shape[1] * scale_percent / 100)
        height = int(image.shape[0] * scale_percent / 100)
        dim = (width, height)

        # 进行等比缩放
        resized_image = cv2.resize(image, dim, interpolation=cv2.INTER_LINEAR)

        # 保存缩放后的图像
        cv2.imwrite(self.image_path, resized_image)
        return scale_percent / 100

    # 进行OCR识别
    def __ocr_read_wording(self):
        # 实例化读取器
        reader = easyocr.Reader(self.model)
        # 读取图像
        result = reader.readtext(self.image_path)
        return result

    # 处理坐标数据
    def __handle_coordinate_data(self, coordinate_set, target_wording):
        search_res_coordinate = None
        logger.info(coordinate_set)
        for item in coordinate_set:
            if item[1] == target_wording:
                search_res_coordinate = item
                break
        if search_res_coordinate is None or search_res_coordinate[2] < self.grade:
            logger.warning(
                '没有搜索到元素「{0}」或元素「{0}」置信度过低'.format(target_wording))
            return False
        else:
            logger.info('识别到元素「{}」, 置信度为：{}'.format(
                search_res_coordinate[1], search_res_coordinate[2]))
            coordinates = search_res_coordinate[0]
            logger.info(coordinates)
            x_coordinate = coordinates[0][0] + \
                (coordinates[1][0] - coordinates[0][0]) / 2
            y_coordinate = coordinates[0][1] + \
                (coordinates[2][1] - coordinates[1][1]) / 2
            logger.info("X坐标：{}，Y坐标：{}".format(
                x_coordinate, y_coordinate))
            return x_coordinate, y_coordinate

    # 获取坐标信息
    def get_coordinate(self, target_wording: str):
        """get_coordinate 获取指定文字的坐标
        Args:
            target_wording (str): 目标文字
        Returns:
            tuple: x轴坐标 & y轴坐标
        """
        logger.info('开始进行OCR识别🔍')
        start = time.time()
        scale_percent = self.__resize_image()
        coordinate_set = self.__ocr_read_wording()
        res = self.__handle_coordinate_data(coordinate_set, target_wording)
        if res is not False:
            x, y = res
            res = x / scale_percent, y / scale_percent
        end = time.time()
        logger.info(f"识别耗时: {end -start}s")
        return res


if __name__ == '__main__':
    OCRDiscern("20230904115549_test_jpg.jpg").get_coordinate("校园场馆")













import os
import re
import time
import subprocess
import requests
import six
import uiautomator2 as u2
import shlex

from kuto.utils.log import logger
from kuto.utils.common import screenshot_util
from kuto.utils.exceptions import KError
from kuto.core.android.common import get_connected_adr_devices


class AndroidDriver(object):

    def __init__(self, device_id=None, pkg_name=None):
        logger.info("初始化安卓驱动")
        self.pkg_name = pkg_name
        if not self.pkg_name:
            raise KError('应用包名不能为空')
        self.device_id = device_id
        if not self.device_id:
            # 未传入设备id时，获取已连接设备
            self.device_id = get_connected_adr_devices()[0]

        self.d = u2.connect(self.device_id)

        if not self.d.alive:
            """判断uiautomator服务是否正常运行，否则重启它"""
            logger.info("uiautomator异常，进行重启！！！")
            self.d.healthcheck()
        else:
            logger.info("uiautomator已就绪")

    @property
    def app_info(self):
        logger.info("获取应用信息")
        info = self.d.app_info(self.pkg_name)
        logger.info(info)
        return info

    @property
    def device_info(self):
        logger.info(f"获取设备信息")
        info = self.d.device_info
        logger.info(info)
        return info

    @property
    def page_content(self):
        logger.info("获取页面xml")
        info = self.d.dump_hierarchy()
        logger.info(info)
        return info

    def assert_act(self, activity_name: str, timeout=5):
        logger.info(f"断言 activity 等于 {activity_name}")
        assert self.d.wait_activity(activity_name, timeout=timeout)

    def uninstall_app(self):
        logger.info(f"卸载应用")
        self.d.app_uninstall(self.pkg_name)

    @staticmethod
    def download_apk(src):
        """下载安装包"""
        start = time.time()
        if isinstance(src, six.string_types):
            if re.match(r"^https?://", src):
                logger.info(f'下载中...')
                file_path = os.path.join(os.getcwd(), src.split('/')[-1])
                r = requests.get(src, stream=True)
                if r.status_code != 200:
                    raise IOError(
                        "Request URL {!r} status_code {}".format(src, r.status_code))
                with open(file_path, 'wb') as f:
                    f.write(r.content)
                end = time.time()
                logger.info(f'下载成功: {file_path}，耗时: {end - start}s')
                return file_path
            elif os.path.isfile(src):
                return src
            else:
                raise IOError("static {!r} not found".format(src))

    def install_app(self, apk_path, auth=True, new=True, helper: list = None):
        """
        安装应用，push改成adb命令之后暂时无法支持远程手机调用
        @param apk_path: 安装包链接，支持本地路径以及http路径
        @param auth: 是否进行授权
        @param new: 是否先卸载再安装
        @param helper：install命令后的各品牌机型适配
        [
            ["assert", {"text": "未发现风险"}],
            ["click", {"text": "继续安装"}],
            ["click", {"text": "完成"}],
            ["input", {"resourceId": "xxx"}, "yyy"]
        ]
        """
        start = time.time()
        logger.info(f"安装应用: {apk_path}")
        # 卸载
        if new is True:
            self.uninstall_app()

        # 下载
        try:
            source = self.download_apk(apk_path)
        except:
            raise KeyError("下载apk失败")

        # 安装
        try:
            cmd_list = ['adb', 'install', "-r", "-t", source]
            if auth is True:
                cmd_list.insert(4, '-g')
            logger.debug(f"{' '.join(cmd_list)}")
            process = subprocess.Popen(cmd_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if helper is None:
                # 等待子进程执行完成
                process.wait()
        except:
            raise KeyError("adb安装失败")

        # 各品牌机型适配
        try:
            if helper is not None:
                for step in helper:
                    method = step[0]
                    loc = step[1]
                    if method == "click":
                        self.d(**loc).click()
                    elif method == "assert":
                        assert self.d(**loc).wait(timeout=10)
                    elif method == "input":
                        content = step[2]
                        self.d(**loc).send_keys(content)
                    else:
                        raise KeyError("不支持的方法")
        except:
            raise KeyError("安装过程处理失败")

        # 删除下载的安装包
        if 'http' in apk_path:
            os.remove(source)

        end = time.time()
        logger.info(f'安装成功，耗时: {end - start}s')

    def start_app(self, stop=True):
        """启动应用
        @param stop: 是否先关闭应用再启动
        """
        logger.info(f"启动应用")
        self.d.app_start(self.pkg_name, stop=stop, use_monkey=True)

    def stop_app(self):
        logger.info("关闭应用")
        self.d.app_stop(self.pkg_name)

    def screenshot(self, file_name=None, position: str = None):
        return screenshot_util(self.d, file_name=file_name, position=position)

    def back(self):
        logger.info("返回上一页")
        self.d.press('back')

    def click(self, x, y):
        logger.info(f"点击坐标: {x}, {y}")
        self.d.click(x, y)

    def click_alerts(self, alert_list: list):
        logger.info(f"点击弹窗: {alert_list}")
        with self.d.watch_context() as ctx:
            for alert in alert_list:
                ctx.when(alert).click()
            ctx.wait_stable()

    def swipe(self, direction: str = None):
        logger.info(f"swipe {direction}")
        key_range = ["left", "right", "up", "down"]
        if direction not in key_range:
            raise KeyError(f"direction取值只能是 {key_range} 其中一个")
        self.d.swipe_ext(direction)


if __name__ == '__main__':
    driver1 = AndroidDriver(
        device_id="UJK0220521066836",
        pkg_name="com.qizhidao.clientapp"
    )
    # helper = [
    #     {"method": "assert", "loc": "未发现风险"},
    #     {"method": "click", "loc": "继续安装"},
    #     {"method": "click", "loc": "完成"}
    # ]
    helper1 = [
        ["assert", {"text": "未发现风险"}],
        ["click", {"text": "继续安装"}],
        ["click", {"text": "完成"}],
    ]
    driver1.install_app(
        apk_path="http://maven.qizhidao.net:8081/repository/packages-host/qzd/qizhidao_app_android/dev-work/jenkins_771/app_qzd_v4.4.2_80403001_2023-07-26_arm64-v8a_pre_release.apk")
    driver1.start_app()

















import shutil
import subprocess
import time
import wda

from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.utils.common import screenshot_util
from kuto.core.ios.common import TideviceUtil, get_connected_ios_devices


def _start_wda_xctest(udid: str, port, wda_bundle_id=None):
    xctool_path = shutil.which("tidevice")
    args = []
    if udid:
        args.extend(["-u", udid])
    args.append("wdaproxy")
    args.extend(["--port", str(port)])
    if wda_bundle_id is not None:
        args.extend(["-B", wda_bundle_id])
    p = subprocess.Popen([xctool_path] + args)
    time.sleep(3)
    if p.poll() is not None:
        raise KError("wda启动失败，可能是手机未连接")


class IosDriver(object):

    def __init__(self, device_id: str = None, pkg_name: str = None):
        """device_id可以是udid，也可以是wda服务url"""
        logger.info("初始化ios驱动")
        if not pkg_name:
            raise KError('应用包名不能为空')
        self.pkg_name = pkg_name

        if device_id is None:
            self.device_id = get_connected_ios_devices()[0]
            self.port = self.device_id.split("-")[0][-4:]
            self.wda_url = f"http://localhost:{self.port}"
        else:
            self.device_id = device_id
            if 'http' in self.device_id:
                self.wda_url = self.device_id
            else:
                self.port = self.device_id.split("-")[0][-4:]
                self.wda_url = f"http://localhost:{self.port}"

        self.d = wda.Client(self.wda_url)

        # check if wda is ready
        if self.d.is_ready():
            logger.info('wda已就绪')
        else:
            if 'http' in self.device_id:
                raise KError("wda异常，请确认服务已正常启动！！！")
            else:
                logger.info('wda未就绪, 现在启动')
                _start_wda_xctest(self.device_id, port=self.port)
                if self.d.is_ready():
                    logger.info('wda启动成功')
                else:
                    raise KError('wda启动失败，可能是WebDriverAgent APP端证书失效!')

    @property
    def device_info(self):
        logger.info("获取设备信息")
        info = self.d.device_info()
        logger.info(info)
        return info

    @property
    def current_app(self):
        logger.info("获取当前应用")
        info = self.d.app_current()
        logger.info(info)
        return info

    @property
    def page_content(self):
        logger.info('获取页面xml内容')
        page_source = self.d.source(accessible=False)
        logger.info(page_source)
        return page_source

    def install_app(self, ipa_url, new=True):
        """安装应用
        @param ipa_url: ipa链接
        @param new: 是否先卸载
        @return:
        """
        logger.info(f"安装应用: {ipa_url}")
        if new is True:
            self.uninstall_app()

        TideviceUtil.install_app(self.device_id, ipa_url)

    def uninstall_app(self):
        logger.info(f"卸载应用: {self.pkg_name}")
        TideviceUtil.uninstall_app(self.device_id, self.pkg_name)

    def start_app(self, stop=True):
        """启动应用
        @param stop: 是否先停止应用
        """
        logger.info(f"启动应用: {self.pkg_name}")
        if stop is True:
            self.d.app_terminate(self.pkg_name)
        self.d.app_start(self.pkg_name)

    def stop_app(self):
        logger.info(f"停止应用: {self.pkg_name}")
        self.d.app_terminate(self.pkg_name)

    def back(self):
        """返回上一页"""
        logger.info("返回上一页")
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    def click(self, x, y):
        logger.info(f"点击坐标: {x}, {y}")
        self.d.click(x, y)

    def screenshot(self, file_name=None, position=None):
        return screenshot_util(self.d, file_name=file_name, position=position)

    def click_alerts(self, alert_list: list):
        """点击弹窗"""
        logger.info(f"点击弹窗: {alert_list}")
        try:
            self.d.alert.click(alert_list)
        except:
            pass

    def swipe(self, direction: str = None):
        logger.info(f"swipe {direction}")
        key_range = ["left", "right", "up", "down"]
        if direction not in key_range:
            raise KeyError(f"direction取值只能是 {key_range} 其中一个")
        if direction == "left":
            self.d.swipe_left()
        elif direction == "right":
            self.d.swipe_right()
        elif direction == "up":
            self.d.swipe_up()
        elif direction == "down":
            self.d.swipe_down()


if __name__ == '__main__':
    driver1 = IosDriver(
        device_id="00008101-000E646A3C29003A",
        pkg_name="com.qizhidao.company"
    )
    driver1.start_app()












"""
@Author: kang.yang
@Date: 202
3/8/1 16:27
"""
from kuto import Page, IosElem as Elem


"""
定位方式：优先选择label
name: 根据name属性进行定位
label: 根据label属性进行定位
value: 根据value属性进行定位
text: 根据文本属性进行定位，集合和label、value等文本属性的内容
className: 根据className属性进行定位
xpath: 根据xpath进行定位
index: 获取到定位到的第index个元素
"""


class IndexPage(Page):
    """首页"""
    adBtn = Elem(label='close white big', desc='广告关闭按钮')
    myTab = Elem(label='我的', desc='我的tab')


class MyPage(Page):
    """我的页"""
    settingBtn = Elem(label='settings navi', desc='设置按钮')


class SettingPage(Page):
    """设置页"""
    about = Elem(text="关于企知道", desc='关于企知道文本')



import inspect
import platform

from selenium.webdriver import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.select import Select
from selenium.webdriver.support.wait import WebDriverWait

from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.core.web.selenium_driver import SeleniumDriver


class SeleElem:
    """
    通过selenium定位的web元素
    """

    def __init__(self,
                 driver: SeleniumDriver = None,
                 xpath: str = None,
                 css: str = None
                 ):
        """

        @param driver: 浏览器驱动
        @param xpath: 根据xpath进行定位
        @param css: 根据css selector进行定位
        """
        self._driver = driver

        if xpath:
            self.loc = (By.XPATH, xpath)
        if css:
            self.loc = (By.CSS_SELECTOR, css)

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self._driver = instance.driver
        return self

    def _wait(self, timeout=3):
        try:
            WebDriverWait(self._driver.d, timeout=timeout).until(
                EC.visibility_of_element_located(self.loc)
            )
            return True
        except Exception:
            return False

    # def _find_element(self, retry=3, timeout=5):
    #     cur_retry = retry
    #     while not self._wait(timeout=timeout):
    #         if cur_retry > 0:
    #             logger.warning(f"第{retry-cur_retry+1}次重试，查找元素： {self.loc}")
    #             cur_retry -= 1
    #         else:
    #             frame = inspect.currentframe().f_back
    #             caller = inspect.getframeinfo(frame)
    #             logger.warning(
    #                 f"【{caller.function}:{caller.lineno}】Not found element {self.loc}"
    #             )
    #             return None
    #     elements = self._driver.d.find_elements(*self.loc)
    #     return elements

    def _find_element(self, timeout=5):
        logger.info(f'try find {self.loc}')
        if self._wait(timeout):
            return self._driver.d.find_elements(*self.loc)
        else:
            return None

    def get_elements(self, timeout=3):
        elements = self._find_element(timeout=timeout)
        if elements is None:
            raise KError(f"[控件 {self.loc} 定位失败]")
        self._driver.screenshot("loc")
        return elements

    def get_element(self, timeout=3):
        elements = self.get_elements(timeout=timeout)
        return elements[0]

    @property
    def display(self):
        logger.info(f"get is_displayed")
        displayed = self.get_element().is_displayed()
        return displayed

    @property
    def text(self):
        logger.info(f"get text")
        element = self.get_element()
        text = element.text
        return text

    def get_attr(self, attr_name):
        logger.info(f"get {attr_name}")
        value = self.get_element().get_attribute(attr_name)
        return value

    def exists(self, timeout=3):
        logger.info(f"if exists")
        element = self._find_element(timeout=timeout)
        if element is None:
            return False
        return True

    def click(self, timeout=5):
        logger.info(f"click")
        self.get_element(timeout=timeout).click()

    def click_exists(self, timeout=1):
        logger.info(f"click if exists")
        if self.exists(timeout=timeout):
            self.click()

    def set_text(self, text):
        logger.info(f"input text")
        self.get_element().send_keys(text)

    def clear_text(self):
        logger.info(f"clear")
        self.get_element().clear()

    def enter(self):
        logger.info(f"enter")
        self.get_element().send_keys(Keys.ENTER)


if __name__ == '__main__':
    pass








"""
@Author: kang.yang
@Date: 2023/8/1 11:53
"""
from kuto import Page, AdrElem as Elem

"""
定位方式：优先选择rid
rid: 根据resourceId进行定位
text：根据text属性进行定位
className：根据className属性进行定位
xpath：根据xpath进行定位
index：获取定位到的第index个元素
"""


class HomePage(Page):
    """APP首页"""
    adBtn = Elem(rid='bottom_btn', desc='广告关闭按钮')
    myTab = Elem(text='我的', desc='我的tab')
    spaceTab = Elem(text='科创空间', desc='科创空间tab')


class MyPage(Page):
    """我的页"""
    settingBtn = Elem(rid='me_top_bar_setting_iv', desc='设置按钮')


class SettingPage(Page):
    """设置页"""
    title = Elem(rid='tv_actionbar_title', desc='设置页标题')
    agreementText = Elem(rid='agreement_tv_2', desc='服务协议链接')



import time

from kuto.core.ios.driver import IosDriver
from kuto.utils.exceptions import KError
from kuto.utils.log import logger
from kuto.core.image.driver import ImageDiscern
from kuto.utils.common import draw_red_by_rect


class ImageElem(object):
    """图像识别定位"""

    def __init__(self,
                 driver=None,
                 desc: str = None,
                 image: str = None,
                 debug: bool = False):
        self.driver = driver
        if not desc:
            raise KError("元素描述不能为空")
        else:
            self._desc = desc
        self.target_image = image
        self._debug = debug

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def exists(self, retry=3, timeout=3, grade=0.9, gauss_num=111):
        logger.info(f'图像识别判断: {self._desc} 是否存在')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i + 1}次查找:')
            source_image = self.driver.screenshot(self._desc)
            res = ImageDiscern(self.target_image, source_image, grade, gauss_num).get_coordinate()
            logger.debug(res)
            if isinstance(res, tuple):
                return True
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'图像识别定位失败-{self._desc}')
            return False

    def click(self, retry=3, timeout=3, grade=0.9, gauss_num=111):
        logger.info(f'图像识别点击图片: {self._desc}')
        time.sleep(3)
        for i in range(retry):
            logger.info(f'第{i + 1}次查找:')
            source_image = self.driver.screenshot(self._desc)
            res = ImageDiscern(self.target_image, source_image, grade, gauss_num).get_coordinate()
            if isinstance(res, tuple):
                logger.info(f'识别坐标为: {res}')
                x, y = res[0], res[1]
                if isinstance(self.driver, IosDriver):
                    x, y = int(x/self.driver.d.scale), int(y/self.driver.d.scale)
                if self._debug is True:
                    file_path = self.driver.screenshot(f'图像识别定位成功-{self._desc}')
                    draw_red_by_rect(file_path, (int(x) - 100, int(y) - 100, 200, 200))
                self.driver.click(x, y)
                return
            time.sleep(timeout)
        else:
            self.driver.screenshot(f'图像识别定位失败-{self._desc}')
            raise Exception('未识别到图片，无法进行点击')


if __name__ == '__main__':
    from kuto.core.android.driver import AndroidDriver

    driver = AndroidDriver(
        device_id='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )




import kuto


if __name__ == '__main__':
    # 执行多个用例文件，主程序入口

    # 执行接口用例
    kuto.main(
        platform="api",
        host='https://app-pre.qizhidao.com',
        path='tests/test_api.py'
    )


